@@ -1,7 +1,7 @@
 // drumkv1_config.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,7 +28,7 @@
 
 #define DRUMKV1_SUBTITLE    "an old-school drum-kit sampler."
 #define DRUMKV1_WEBSITE     "https://drumkv1.sourceforge.io"
-#define DRUMKV1_COPYRIGHT   "Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved."
+#define DRUMKV1_COPYRIGHT   "Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved."
 
 #define DRUMKV1_DOMAIN      "rncbc.org"
 
@@ -67,6 +67,9 @@ public:
 	// Special time-formatted spinbox option.
 	int iFrameTimeFormat;
 
+	// Default randomize factor (percent).
+	float fRandomizePercent;
+
 	// Whether to display GM Standard drum-note/key names.
 	bool bUseGMDrumNames;
 
@@ -78,7 +81,8 @@ public:
 	// Run-time special non-persistent options.
 	bool bDontUseNativeDialogs;
 
-	// Custom widget style theme.
+	// Custom color palette/widget style themes.
+	QString sCustomColorTheme;
 	QString sCustomStyleTheme;
 
 	// Micro-tuning options.
