@@ -1,7 +1,7 @@
 // drumkv1_controls.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -342,7 +342,7 @@ public:
 			xrpn_item& item = get_item(channel);
 			if (item.is_any() && item.type() != drumkv1_controls::RPN)
 				enqueue(item);
-			if (item.is_status() // RPN NULL [MSB]
+			if (item.is_status() // RPN nullptr [MSB]
 				&& item.type() == drumkv1_controls::RPN && item.is_param_lsb()
 				&& item.param_lsb() == 0x7f && event.value == 0x7f) {
 				item.clear();
@@ -366,7 +366,7 @@ public:
 			xrpn_item& item = get_item(channel);
 			if (item.is_any() && item.type() != drumkv1_controls::RPN)
 				enqueue(item);
-			if (item.is_status() // RPN NULL [LSB]
+			if (item.is_status() // RPN nullptr [LSB]
 				&& item.type() == drumkv1_controls::RPN && item.is_param_msb()
 				&& item.param_msb() == 0x7f && event.value == 0x7f) {
 				item.clear();
