@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -65,7 +65,7 @@ protected:
 	void run()
 	{
 		snd_seq_t *seq = m_drumk->alsa_seq();
-		if (seq == NULL)
+		if (seq == nullptr)
 			return;
 
 		m_running = true;
@@ -82,7 +82,7 @@ protected:
 		while (m_running && poll_rc >= 0) {
 			poll_rc = ::poll(pfds, nfds, 200);
 			while (poll_rc > 0) {
-				snd_seq_event_t *ev = NULL;
+				snd_seq_event_t *ev = nullptr;
 				snd_seq_event_input(seq, &ev);
 				m_drumk->alsa_capture(ev);
 			//	snd_seq_free_event(ev);
@@ -123,6 +123,15 @@ static int drumkv1_jack_buffer_size ( jack_nframes_t nframes, void *arg )
 }
 
 
+//----------------------------------------------------------------------
+// JACK on-shutdown callback.
+
+static void drumkv1_jack_on_shutdown ( void *arg )
+{
+	static_cast<drumkv1_jack *> (arg)->shutdown();
+}
+
+
 #ifdef CONFIG_JACK_SESSION
 
 #include <jack/session.h>
@@ -151,27 +160,27 @@ static void drumkv1_jack_session_event (
 
 drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 {
-	m_client = NULL;
+	m_client = nullptr;
 
 	m_activated = false;
 
-	m_audio_ins = NULL;
-	m_audio_outs = NULL;
+	m_audio_ins = nullptr;
+	m_audio_outs = nullptr;
 
-	m_ins = m_outs = NULL;
+	m_ins = m_outs = nullptr;
 
 	::memset(m_params, 0, drumkv1::NUM_PARAMS * sizeof(float));
 
 #ifdef CONFIG_JACK_MIDI
-	m_midi_in = NULL;
+	m_midi_in = nullptr;
 #endif
 #ifdef CONFIG_ALSA_MIDI
-	m_alsa_seq     = NULL;
+	m_alsa_seq     = nullptr;
 //	m_alsa_client  = -1;
 	m_alsa_port    = -1;
-	m_alsa_decoder = NULL;
-	m_alsa_buffer  = NULL;
-	m_alsa_thread  = NULL;
+	m_alsa_decoder = nullptr;
+	m_alsa_buffer  = nullptr;
+	m_alsa_thread  = nullptr;
 #endif
 
 	drumkv1::programs()->enabled(true);
@@ -289,8 +298,8 @@ void drumkv1_jack::open ( const char *client_id )
 	}
 
 	// open client
-	m_client = ::jack_client_open(client_id, JackNullOption, NULL);
-	if (m_client == NULL)
+	m_client = ::jack_client_open(client_id, JackNullOption, nullptr);
+	if (m_client == nullptr)
 		return;
 
 	// set sample rate
@@ -311,11 +320,11 @@ void drumkv1_jack::open ( const char *client_id )
 		::snprintf(port_name, sizeof(port_name), "in_%d", k + 1);
 		m_audio_ins[k] = ::jack_port_register(m_client,
 			port_name, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
-		m_ins[k] = NULL;
+		m_ins[k] = nullptr;
 		::snprintf(port_name, sizeof(port_name), "out_%d", k + 1);
 		m_audio_outs[k] = ::jack_port_register(m_client,
 			port_name, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-		m_outs[k] = NULL;
+		m_outs[k] = nullptr;
 	}
 
 	// register midi port
@@ -324,12 +333,12 @@ void drumkv1_jack::open ( const char *client_id )
 		"in", JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
 #endif
 #ifdef CONFIG_ALSA_MIDI
-	m_alsa_seq     = NULL;
+	m_alsa_seq     = nullptr;
 //	m_alsa_client  = -1;
 	m_alsa_port    = -1;
-	m_alsa_decoder = NULL;
-	m_alsa_buffer  = NULL;
-	m_alsa_thread  = NULL;
+	m_alsa_decoder = nullptr;
+	m_alsa_buffer  = nullptr;
+	m_alsa_thread  = nullptr;
 	// open alsa sequencer client...
 	if (snd_seq_open(&m_alsa_seq, "hw", SND_SEQ_OPEN_INPUT, 0) >= 0) {
 		snd_seq_set_client_name(m_alsa_seq, client_id);
@@ -351,6 +360,9 @@ void drumkv1_jack::open ( const char *client_id )
 	jack_set_buffer_size_callback(m_client,
 		drumkv1_jack_buffer_size, this);
 
+	jack_on_shutdown(m_client,
+		drumkv1_jack_on_shutdown, this);
+
 	// set process callbacks...
 	::jack_set_process_callback(m_client,
 		drumkv1_jack_process, this);
@@ -394,15 +406,15 @@ void drumkv1_jack::close (void)
 	if (m_alsa_seq) {
 		if (m_alsa_thread) {
 			delete m_alsa_thread;
-			m_alsa_thread = NULL;
+			m_alsa_thread = nullptr;
 		}
 		if (m_alsa_buffer) {
 			::jack_ringbuffer_free(m_alsa_buffer);
-			m_alsa_buffer = NULL;
+			m_alsa_buffer = nullptr;
 		}
 		if (m_alsa_decoder) {
 			snd_midi_event_free(m_alsa_decoder);
-			m_alsa_decoder = NULL;
+			m_alsa_decoder = nullptr;
 		}
 		if (m_alsa_port >= 0) {
 			snd_seq_delete_simple_port(m_alsa_seq, m_alsa_port);
@@ -410,18 +422,18 @@ void drumkv1_jack::close (void)
 		}
 		snd_seq_close(m_alsa_seq);
 	//	m_alsa_client = -1;
-		m_alsa_seq = NULL;
+		m_alsa_seq = nullptr;
 	}
 #endif
 
-	if (m_client == NULL)
+	if (m_client == nullptr)
 		return;
 
 #ifdef CONFIG_JACK_MIDI
 	// unregister midi port
 	if (m_midi_in) {
 		::jack_port_unregister(m_client, m_midi_in);
-		m_midi_in = NULL;
+		m_midi_in = nullptr;
 	}
 #endif
 
@@ -431,39 +443,39 @@ void drumkv1_jack::close (void)
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		if (m_audio_outs && m_audio_outs[k]) {
 			::jack_port_unregister(m_client, m_audio_outs[k]);
-			m_audio_outs[k] = NULL;
+			m_audio_outs[k] = nullptr;
 		}
 		if (m_outs && m_outs[k])
-			m_outs[k] = NULL;
+			m_outs[k] = nullptr;
 		if (m_audio_ins && m_audio_ins[k]) {
 			::jack_port_unregister(m_client, m_audio_ins[k]);
-			m_audio_ins[k] = NULL;
+			m_audio_ins[k] = nullptr;
 		}
 		if (m_ins && m_ins[k])
-			m_ins[k] = NULL;
+			m_ins[k] = nullptr;
 	}
 
 	if (m_outs) {
 		delete [] m_outs;
-		m_outs = NULL;
+		m_outs = nullptr;
 	}
 	if (m_ins) {
 		delete [] m_ins;
-		m_ins = NULL;
+		m_ins = nullptr;
 	}
 
 	if (m_audio_outs) {
 		delete [] m_audio_outs;
-		m_audio_outs = NULL;
+		m_audio_outs = nullptr;
 	}
 	if (m_audio_ins) {
 		delete [] m_audio_ins;
-		m_audio_ins = NULL;
+		m_audio_ins = nullptr;
 	}
 
 	// close client
 	::jack_client_close(m_client);
-	m_client = NULL;
+	m_client = nullptr;
 }
 
 
@@ -478,10 +490,10 @@ snd_seq_t *drumkv1_jack::alsa_seq (void) const
 // alsa event capture.
 void drumkv1_jack::alsa_capture ( snd_seq_event_t *ev )
 {
-	if (m_alsa_decoder == NULL)
+	if (m_alsa_decoder == nullptr)
 		return;
 
-	if (ev == NULL)
+	if (ev == nullptr)
 		return;
 
 	// ignored events...
@@ -617,6 +629,33 @@ void drumkv1_jack::updateSample (void)
 }
 
 
+void drumkv1_jack::updateTuning (void)
+{
+	drumkv1::resetTuning();
+}
+
+
+void drumkv1_jack::shutdown (void)
+{
+	drumkv1_jack_application *pApp = drumkv1_jack_application::getInstance();
+	if (pApp)
+		pApp->shutdown();
+}
+
+
+void drumkv1_jack::shutdown_close (void)
+{
+	m_activated = false;
+
+	if (m_client) {
+		::jack_client_close(m_client);
+		m_client = nullptr;
+	}
+
+	close();
+}
+
+
 //-------------------------------------------------------------------------
 // drumkv1_jack_application -- Singleton application instance.
 //
@@ -644,7 +683,7 @@ void drumkv1_jack::updateSample (void)
 static int g_fdSigterm[2] = { -1, -1 };
 
 // Unix SIGTERM signal handler.
-void drumkv1_sigterm_handler ( int /*signo*/ )
+static void drumkv1_sigterm_handler ( int /*signo*/ )
 {
 	char c = 1;
 
@@ -656,10 +695,10 @@ void drumkv1_sigterm_handler ( int /*signo*/ )
 
 // Constructor.
 drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
-	: QObject(NULL), m_pApp(NULL), m_bGui(true),
-		m_pDrumk(NULL), m_pWidget(NULL)
+	: QObject(nullptr), m_pApp(nullptr), m_bGui(true),
+		m_pDrumk(nullptr), m_pWidget(nullptr)
 	  #ifdef CONFIG_NSM
-		, m_pNsmClient(NULL)
+		, m_pNsmClient(nullptr)
 	  #endif
 {
 #ifdef Q_WS_X11
@@ -674,53 +713,67 @@ drumkv1_jack_application::drumkv1_jack_application ( int& argc, char **argv )
 			m_bGui = false;
 	}
 
-	if (m_bGui)
-		m_pApp = new QApplication(argc, argv);
-	else
+	if (m_bGui) {
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+		QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+	#endif
+		QApplication *pApp = new QApplication(argc, argv);
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
+		pApp->setApplicationDisplayName(DRUMKV1_TITLE);
+		//	DRUMKV1_TITLE " - " + QObject::tr(DRUMKV1_SUBTITLE));
+	#endif
+		m_pApp = pApp;
+	} else {
 		m_pApp = new QCoreApplication(argc, argv);
+	}
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
+	m_pApp->setApplicationName(DRUMKV1_TITLE);
+#endif
+	
 #ifdef HAVE_SIGNAL_H
 
-	// set to ignore any fatal "Broken pipe" signals.
+	// Set to ignore any fatal "Broken pipe" signals.
 	::signal(SIGPIPE, SIG_IGN);
 
-	// initialize file descriptors for SIGTERM socket notifier.
+	// Initialize file descriptors for SIGTERM socket notifier.
 	::socketpair(AF_UNIX, SOCK_STREAM, 0, g_fdSigterm);
-	m_pSigtermNotifier = new QSocketNotifier(
-		g_fdSigterm[1], QSocketNotifier::Read, this);
+	m_pSigtermNotifier
+		= new QSocketNotifier(g_fdSigterm[1], QSocketNotifier::Read, this);
 
 	QObject::connect(m_pSigtermNotifier,
 		SIGNAL(activated(int)),
-		SLOT(sigterm_handler()));
+		SLOT(handle_sigterm()));
 
-	// install SIGTERM signal handler.
-	struct sigaction term;
-	term.sa_handler = drumkv1_sigterm_handler;
-	::sigemptyset(&term.sa_mask);
-	term.sa_flags = 0;
-	term.sa_flags |= SA_RESTART;
-	::sigaction(SIGTERM, &term, NULL);
-	::sigaction(SIGINT,  &term, NULL);
+	// Install SIGTERM signal handler.
+	struct sigaction sigterm;
+	sigterm.sa_handler = drumkv1_sigterm_handler;
+	sigemptyset(&sigterm.sa_mask);
+	sigterm.sa_flags = 0;
+	sigterm.sa_flags |= SA_RESTART;
+	::sigaction(SIGTERM, &sigterm, nullptr);
+	::sigaction(SIGQUIT, &sigterm, nullptr);
 
-	// ignore SIGHUP signal.
-	struct sigaction hup;
-	hup.sa_handler = SIG_IGN;
-	::sigemptyset(&hup.sa_mask);
-	hup.sa_flags = 0;
-	hup.sa_flags |= SA_RESTART;
-	::sigaction(SIGHUP, &hup, NULL);
+	// Ignore SIGHUP/SIGINT signals.
+	::signal(SIGHUP, SIG_IGN);
+	::signal(SIGINT, SIG_IGN);
 
 #else
 
-	m_pSigtermNotifier = NULL;
+	m_pSigtermNotifier = nullptr;
 
 #endif	// !HAVE_SIGNAL_H
+
+	// Pseudo-singleton instance.
+	g_pInstance = this;
 }
 
 
 // Destructor.
 drumkv1_jack_application::~drumkv1_jack_application (void)
 {
+	g_pInstance = nullptr;
+
 #ifdef HAVE_SIGNAL_H
 	if (m_pSigtermNotifier) delete m_pSigtermNotifier;
 #endif
@@ -771,7 +824,7 @@ bool drumkv1_jack_application::parse_args (void)
 // Startup methods.
 bool drumkv1_jack_application::setup (void)
 {
-	if (m_pApp == NULL)
+	if (m_pApp == nullptr)
 		return false;
 
 	if (!parse_args()) {
@@ -779,6 +832,10 @@ bool drumkv1_jack_application::setup (void)
 		return false;
 	}
 
+	QObject::connect(this,
+		SIGNAL(shutdown_signal()),
+		SLOT(shutdown_slot()));
+
 	m_pDrumk = new drumkv1_jack();
 
 	if (m_bGui) {
@@ -838,10 +895,10 @@ int drumkv1_jack_application::exec (void)
 
 void drumkv1_jack_application::openSession (void)
 {
-	if (m_pDrumk == NULL)
+	if (m_pDrumk == nullptr)
 		return;
 
-	if (m_pNsmClient == NULL)
+	if (m_pNsmClient == nullptr)
 		return;
 
 	if (!m_pNsmClient->is_active())
@@ -886,10 +943,10 @@ void drumkv1_jack_application::openSession (void)
 
 void drumkv1_jack_application::saveSession (void)
 {
-	if (m_pDrumk == NULL)
+	if (m_pDrumk == nullptr)
 		return;
 
-	if (m_pNsmClient == NULL)
+	if (m_pNsmClient == nullptr)
 		return;
 
 	if (!m_pNsmClient->is_active())
@@ -914,7 +971,7 @@ void drumkv1_jack_application::saveSession (void)
 
 void drumkv1_jack_application::showSession (void)
 {
-	if (m_pNsmClient == NULL)
+	if (m_pNsmClient == nullptr)
 		return;
 
 	if (!m_pNsmClient->is_active())
@@ -933,7 +990,7 @@ void drumkv1_jack_application::showSession (void)
 
 void drumkv1_jack_application::hideSession (void)
 {
-	if (m_pNsmClient == NULL)
+	if (m_pNsmClient == nullptr)
 		return;
 
 	if (!m_pNsmClient->is_active())
@@ -954,7 +1011,7 @@ void drumkv1_jack_application::hideSession (void)
 #ifdef HAVE_SIGNAL_H
 
 // SIGTERM signal handler.
-void drumkv1_jack_application::sigterm_handler (void)
+void drumkv1_jack_application::handle_sigterm (void)
 {
 	char c;
 
@@ -967,6 +1024,37 @@ void drumkv1_jack_application::sigterm_handler (void)
 #endif	// HAVE_SIGNAL_H
 
 
+// JACK shutdown handlers.
+void drumkv1_jack_application::shutdown (void)
+{
+	emit shutdown_signal();
+}
+
+
+void drumkv1_jack_application::shutdown_slot (void)
+{
+	bool bQuit = true;
+
+	if (m_pDrumk)
+		m_pDrumk->shutdown_close();
+
+	if (m_pWidget)
+		bQuit = m_pWidget->queryClose();
+
+	if (m_pApp && bQuit)
+		m_pApp->quit();
+}
+
+
+// Pseudo-singleton instance.
+drumkv1_jack_application *drumkv1_jack_application::g_pInstance = nullptr;
+
+drumkv1_jack_application *drumkv1_jack_application::getInstance (void)
+{
+	return g_pInstance;
+}
+
+
 //-------------------------------------------------------------------------
 // main
 
