@@ -1,7 +1,7 @@
 // drumkv1_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -67,12 +67,16 @@ public:
 	void sessionEvent(void *pvSessionArg);
 #endif
 
+	void shutdown();
+	void shutdown_close();
+
 protected:
 
 	void updatePreset(bool bDirty);
 
 	void selectSample(int key);
 	void updateSample();
+	void updateTuning();
 
 private:
 
@@ -137,24 +141,33 @@ public:
 	// Facade method.
 	int exec();
 
-#ifdef CONFIG_NSM
+	// JACK shutdown handler.
+	void shutdown();
+
+	// Pseudo-singleton accessor.
+	static drumkv1_jack_application *getInstance();
+
+signals:
+
+	void shutdown_signal();
 
 protected slots:
 
+#ifdef CONFIG_NSM
 	// NSM callback slots.
 	void openSession();
 	void saveSession();
-
 	void hideSession();
 	void showSession();
-
 #endif	// CONFIG_NSM
 
 #ifdef HAVE_SIGNAL_H
 	// SIGTERM signal handler.
-	void sigterm_handler();
+	void handle_sigterm();
 #endif
 
+	void shutdown_slot();
+
 protected:
 
 	// Argument parser method.
@@ -168,6 +181,7 @@ private:
 	// Instance variables.
 	QCoreApplication *m_pApp;
 	bool m_bGui;
+
 	QStringList m_presets;
 
 	drumkv1_jack *m_pDrumk;
@@ -180,6 +194,8 @@ private:
 #ifdef HAVE_SIGNAL_H
 	QSocketNotifier *m_pSigtermNotifier;
 #endif
+
+	static drumkv1_jack_application *g_pInstance;
 };
 
 
