@@ -1,7 +1,7 @@
 // drumkv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -59,6 +59,7 @@
 #include <stdlib.h>
 #include <math.h>
 
+#include <QApplication>
 #include <QDomDocument>
 #include <QFileInfo>
 
@@ -72,7 +73,7 @@ class drumkv1_lv2_map_path : public drumkv1_param::map_path
 public:
 
 	drumkv1_lv2_map_path(const LV2_Feature *const *features)
-		: m_map_path(NULL)
+		: m_map_path(nullptr)
 	{
 		for (int i = 0; features && features[i]; ++i) {
 			if (::strcmp(features[i]->URI, LV2_STATE__mapPath) == 0) {
@@ -138,13 +139,13 @@ drumkv1_lv2::drumkv1_lv2 (
 {
 	::memset(&m_urids, 0, sizeof(m_urids));
 
-	m_urid_map = NULL;
-	m_atom_in  = NULL;
-	m_atom_out = NULL;
-	m_schedule = NULL;
+	m_urid_map = nullptr;
+	m_atom_in  = nullptr;
+	m_atom_out = nullptr;
+	m_schedule = nullptr;
 	m_ndelta   = 0;
 
-	const LV2_Options_Option *host_options = NULL;
+	const LV2_Options_Option *host_options = nullptr;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
 		const LV2_Feature *host_feature = host_features[i];
@@ -169,6 +170,18 @@ drumkv1_lv2::drumkv1_lv2 (
 				    m_urid_map->handle, DRUMKV1_LV2_PREFIX "GEN1_SELECT");
 				m_urids.gen1_update = m_urid_map->map(
 					m_urid_map->handle, DRUMKV1_LV2_PREFIX "GEN1_UPDATE");
+				m_urids.p201_tuning_enabled = m_urid_map->map(
+					m_urid_map->handle, DRUMKV1_LV2_PREFIX "P201_TUNING_ENABLED");
+				m_urids.p202_tuning_refPitch = m_urid_map->map(
+					m_urid_map->handle, DRUMKV1_LV2_PREFIX "P202_TUNING_REF_PITCH");
+				m_urids.p203_tuning_refNote = m_urid_map->map(
+					m_urid_map->handle, DRUMKV1_LV2_PREFIX "P203_TUNING_REF_NOTE");
+				m_urids.p204_tuning_scaleFile = m_urid_map->map(
+					m_urid_map->handle, DRUMKV1_LV2_PREFIX "P204_TUNING_SCALE_FILE");
+				m_urids.p205_tuning_keyMapFile = m_urid_map->map(
+					m_urid_map->handle, DRUMKV1_LV2_PREFIX "P205_TUNING_KEYMAP_FILE");
+				m_urids.tun1_update = m_urid_map->map(
+					m_urid_map->handle, DRUMKV1_LV2_PREFIX "TUN1_UPDATE");
 				m_urids.atom_Blank = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Blank);
 				m_urids.atom_Object = m_urid_map->map(
@@ -177,6 +190,8 @@ drumkv1_lv2::drumkv1_lv2 (
 					m_urid_map->handle, LV2_ATOM__Float);
 				m_urids.atom_Int = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Int);
+				m_urids.atom_Bool = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Bool);
 				m_urids.atom_Path = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Path);
 				m_urids.time_Position = m_urid_map->map(
@@ -249,7 +264,7 @@ drumkv1_lv2::drumkv1_lv2 (
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
-		m_ins[k] = m_outs[k] = NULL;
+		m_ins[k] = m_outs[k] = nullptr;
 }
 
 
@@ -307,7 +322,7 @@ void drumkv1_lv2::run ( uint32_t nframes )
 
 	if (m_atom_in) {
 		LV2_ATOM_SEQUENCE_FOREACH(m_atom_in, event) {
-			if (event == NULL)
+			if (event == nullptr)
 				continue;
 			if (event->body.type == m_urids.midi_MidiEvent) {
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
@@ -330,9 +345,9 @@ void drumkv1_lv2::run ( uint32_t nframes )
 				const LV2_Atom_Object *object
 					= (LV2_Atom_Object *) &event->body;
 				if (object->body.otype == m_urids.time_Position) {
-					LV2_Atom *atom = NULL;
+					LV2_Atom *atom = nullptr;
 					lv2_atom_object_get(object,
-						m_urids.time_beatsPerMinute, &atom, NULL);
+						m_urids.time_beatsPerMinute, &atom, nullptr);
 					if (atom && atom->type == m_urids.atom_Float) {
 						const float host_bpm = ((LV2_Atom_Float *) atom)->body;
 						if (::fabsf(host_bpm - drumkv1::tempo()) > 0.001f)
@@ -343,17 +358,19 @@ void drumkv1_lv2::run ( uint32_t nframes )
 				else 
 				if (object->body.otype == m_urids.patch_Set) {
 					// set property value
-					const LV2_Atom *property = NULL;
-					const LV2_Atom *value = NULL;
+					const LV2_Atom *property = nullptr;
+					const LV2_Atom *value = nullptr;
 					lv2_atom_object_get(object,
 						m_urids.patch_property, &property,
 						m_urids.patch_value, &value, 0);
 					if (property && value && property->type == m_forge.URID) {
 						const uint32_t key = ((const LV2_Atom_URID *) property)->body;
 						const LV2_URID type = value->type;
-						if ((key == m_urids.p101_sample_file ||
-							 key == m_urids.gen1_sample)
-							&& type == m_urids.atom_Path) {
+						if ((key == m_urids.p101_sample_file
+						#if 1//DRUMKV1_LV2_LEGACY
+							 || key == m_urids.gen1_sample
+						#endif
+							) && type == m_urids.atom_Path) {
 							if (m_schedule) {
 								drumkv1_lv2_worker_message mesg;
 								mesg.atom.type = key;
@@ -366,9 +383,11 @@ void drumkv1_lv2::run ( uint32_t nframes )
 							}
 						}
 						else
-						if ((key == m_urids.p102_offset_start ||
-							 key == m_urids.gen1_offset_start)
-							&& type == m_urids.atom_Int) {
+						if ((key == m_urids.p102_offset_start
+						#if 1//DRUMKV1_LV2_LEGACY
+							 || key == m_urids.gen1_offset_start
+						#endif
+							) && type == m_urids.atom_Int) {
 							drumkv1_sample *pSample = drumkv1::sample();
 							if (pSample) {
 								const uint32_t offset_start
@@ -379,9 +398,11 @@ void drumkv1_lv2::run ( uint32_t nframes )
 							}
 						}
 						else
-						if ((key == m_urids.p103_offset_end ||
-							 key == m_urids.gen1_offset_end)
-							&& type == m_urids.atom_Int) {
+						if ((key == m_urids.p103_offset_end
+						#if 1//DRUMKV1_LV2_LEGACY
+							 || key == m_urids.gen1_offset_end
+						#endif
+							) && type == m_urids.atom_Int) {
 							drumkv1_sample *pSample = drumkv1::sample();
 							if (pSample) {
 								const uint32_t offset_start
@@ -391,11 +412,51 @@ void drumkv1_lv2::run ( uint32_t nframes )
 								setOffsetRange(offset_start, offset_end);
 							}
 						}
+						else
+						if (key == m_urids.p201_tuning_enabled
+							&& type == m_urids.atom_Bool) {
+							const uint32_t enabled
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							drumkv1::setTuningEnabled(enabled > 0);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p202_tuning_refPitch
+							&& type == m_urids.atom_Float) {
+							const float refPitch
+								= *(float *) LV2_ATOM_BODY_CONST(value);
+							drumkv1::setTuningRefPitch(refPitch);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p203_tuning_refNote
+							&& type == m_urids.atom_Int) {
+							const uint32_t refNote
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							drumkv1::setTuningRefNote(refNote);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p204_tuning_scaleFile
+							&& type == m_urids.atom_Path) {
+							const char *scaleFile
+								= (const char *) LV2_ATOM_BODY_CONST(value);
+							drumkv1::setTuningScaleFile(scaleFile);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p205_tuning_keyMapFile
+							&& type == m_urids.atom_Path) {
+							const char *keyMapFile
+								= (const char *) LV2_ATOM_BODY_CONST(value);
+							drumkv1::setTuningKeyMapFile(keyMapFile);
+							updateTuning();
+						}
 					}
 				}
 				else
 				if (object->body.otype == m_urids.patch_Get) {
-					// put property values (probably to UI)
+					// put all property values (probably to UI)
 					patch_put(ndelta);
 				}
 			#endif	// CONFIG_LV2_PATCH
@@ -403,7 +464,7 @@ void drumkv1_lv2::run ( uint32_t nframes )
 		}
 		// remember last time for worker response
 		m_ndelta = ndelta;
-	//	m_atom_in = NULL;
+	//	m_atom_in = nullptr;
 	}
 
 	if (nframes > ndelta)
@@ -432,6 +493,42 @@ uint32_t drumkv1_lv2::urid_map ( const char *uri ) const
 }
 
 
+//-------------------------------------------------------------------------
+// drumkv1_lv2 - Instantiation and cleanup.
+//
+
+QApplication *drumkv1_lv2::g_qapp_instance = nullptr;
+unsigned int  drumkv1_lv2::g_qapp_refcount = 0;
+
+
+void drumkv1_lv2::qapp_instantiate (void)
+{
+	if (qApp == nullptr && g_qapp_instance == nullptr) {
+		static int s_argc = 1;
+		static const char *s_argv[] = { __func__, nullptr };
+		g_qapp_instance = new QApplication(s_argc, (char **) s_argv);
+	}
+
+	if (g_qapp_instance)
+		g_qapp_refcount++;
+}
+
+
+void drumkv1_lv2::qapp_cleanup (void)
+{
+	if (g_qapp_instance && --g_qapp_refcount == 0) {
+		delete g_qapp_instance;
+		g_qapp_instance = nullptr;
+	}
+}
+
+
+QApplication *drumkv1_lv2::qapp_instance (void)
+{
+	return g_qapp_instance;
+}
+
+
 //-------------------------------------------------------------------------
 // drumkv1_lv2 - LV2 State interface.
 //
@@ -441,9 +538,11 @@ static LV2_State_Status drumkv1_lv2_state_save ( LV2_Handle instance,
 	uint32_t flags, const LV2_Feature *const *features )
 {
 	drumkv1_lv2 *pPlugin = static_cast<drumkv1_lv2 *> (instance);
-	if (pPlugin == NULL)
+	if (pPlugin == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
+	// Save all state as XML chunk...
+	//
 	const uint32_t key = pPlugin->urid_map(DRUMKV1_LV2_PREFIX "state");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
@@ -460,9 +559,19 @@ static LV2_State_Status drumkv1_lv2_state_save ( LV2_Handle instance,
 	drumkv1_lv2_map_path mapPath(features);
 
 	QDomDocument doc(DRUMKV1_TITLE);
+	QDomElement eState = doc.createElement("state");
+
 	QDomElement eElements = doc.createElement("elements");
 	drumkv1_param::saveElements(pPlugin, doc, eElements, mapPath);
-	doc.appendChild(eElements);
+	eState.appendChild(eElements);
+
+	if (pPlugin->isTuningEnabled()) {
+		QDomElement eTuning = doc.createElement("tuning");
+		drumkv1_param::saveTuning(pPlugin, doc, eTuning);
+		eState.appendChild(eTuning);
+	}
+
+	doc.appendChild(eState);
 
 	const QByteArray data(doc.toByteArray());
 	const char *value = data.constData();
@@ -477,9 +586,11 @@ static LV2_State_Status drumkv1_lv2_state_restore ( LV2_Handle instance,
 	uint32_t flags, const LV2_Feature *const *features )
 {
 	drumkv1_lv2 *pPlugin = static_cast<drumkv1_lv2 *> (instance);
-	if (pPlugin == NULL)
+	if (pPlugin == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
+	// Retrieve all state as XML chunk...
+	//
 	const uint32_t key = pPlugin->urid_map(DRUMKV1_LV2_PREFIX "state");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
@@ -504,16 +615,33 @@ static LV2_State_Status drumkv1_lv2_state_restore ( LV2_Handle instance,
 	if ((flags & (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE)) == 0)
 		return LV2_STATE_ERR_BAD_FLAGS;
 
-	if (value == NULL)
+	if (value == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
 	drumkv1_lv2_map_path mapPath(features);
 
 	QDomDocument doc(DRUMKV1_TITLE);
 	if (doc.setContent(QByteArray(value, size))) {
-		QDomElement eElements = doc.documentElement();
-		if (eElements.tagName() == "elements")
-			drumkv1_param::loadElements(pPlugin, eElements, mapPath);
+		QDomElement eState = doc.documentElement();
+	#if 1//DRUMKV1_LV2_LEGACY
+		if (eState.tagName() == "elements")
+			drumkv1_param::loadElements(pPlugin, eState, mapPath);
+		else
+	#endif
+		if (eState.tagName() == "state") {
+			for (QDomNode nChild = eState.firstChild();
+					!nChild.isNull();
+						nChild = nChild.nextSibling()) {
+				QDomElement eChild = nChild.toElement();
+				if (eChild.isNull())
+					continue;
+				if (eChild.tagName() == "elements")
+					drumkv1_param::loadElements(pPlugin, eChild, mapPath);
+				else
+				if (eChild.tagName() == "tuning")
+					drumkv1_param::loadTuning(pPlugin, eChild);
+			}
+		}
 	}
 
 	pPlugin->reset();
@@ -558,7 +686,7 @@ const LV2_Program_Descriptor *drumkv1_lv2::get_program ( uint32_t index )
 		}
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 void drumkv1_lv2::select_program ( uint32_t bank, uint32_t program )
@@ -575,7 +703,7 @@ void drumkv1_lv2::updatePreset ( bool /*bDirty*/ )
 		drumkv1_lv2_worker_message mesg;
 		mesg.atom.type = m_urids.state_StateChanged;
 		mesg.atom.size = 0; // nothing else matters.
-		mesg.sample.path = NULL;
+		mesg.sample.path = nullptr;
 		m_schedule->schedule_work(
 			m_schedule->handle, sizeof(mesg), &mesg);
 	}
@@ -603,7 +731,19 @@ void drumkv1_lv2::selectSample ( int key )
 		mesg.atom.size = sizeof(mesg.sample);
 		mesg.sample.key = key;
 		m_schedule->schedule_work(
-		    m_schedule->handle, sizeof(mesg), &mesg);
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+void drumkv1_lv2::updateTuning (void)
+{
+	if (m_schedule) {
+		drumkv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.tun1_update;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
 	}
 }
 
@@ -629,13 +769,18 @@ bool drumkv1_lv2::worker_work ( const void *data, uint32_t size )
 	else
 	if (mesg->atom.type == m_urids.gen1_sample) {
 		const int key = drumkv1::currentElement();
-		if (drumkv1::element(key) == NULL) {
+		if (drumkv1::element(key) == nullptr) {
 			drumkv1::addElement(key);
 			drumkv1::setCurrentElementEx(key);
 		}
 		drumkv1::setSampleFile(mesg->sample.path);
 		return true;
 	}
+	else
+	if (mesg->atom.type == m_urids.tun1_update) {
+		drumkv1::resetTuning();
+		return true;
+	}
 
 	return false;
 }
@@ -655,7 +800,7 @@ bool drumkv1_lv2::worker_response ( const void *data, uint32_t size )
 	drumkv1_sched::sync_notify(this, drumkv1_sched::Sample, 0);
 
 #ifdef CONFIG_LV2_PATCH
-	return patch_put(m_ndelta);
+	return patch_put(m_ndelta, mesg->atom.type);
 #else
 	return true;
 #endif
@@ -676,15 +821,16 @@ bool drumkv1_lv2::state_changed (void)
 
 #ifdef CONFIG_LV2_PATCH
 
-bool drumkv1_lv2::patch_put ( uint32_t ndelta )
+bool drumkv1_lv2::patch_put ( uint32_t ndelta, uint32_t type )
 {
 	static char s_szNull[1] = {'\0'};
-	const char *pszSampleFile = NULL;
+
+	if (type == m_urids.patch_Put)
+		type = 0;
+
 	drumkv1_sample *pSample = drumkv1::sample();
-	if (pSample)
-		pszSampleFile = pSample->filename();
-	if (pszSampleFile == NULL)
-		pszSampleFile = s_szNull;
+	if (pSample == nullptr)
+		return false;
 
 	lv2_atom_forge_frame_time(&m_forge, ndelta);
 
@@ -694,12 +840,49 @@ bool drumkv1_lv2::patch_put ( uint32_t ndelta )
 
 	LV2_Atom_Forge_Frame body_frame;
 	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
-	lv2_atom_forge_key(&m_forge, m_urids.p101_sample_file);
-	lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
-	lv2_atom_forge_key(&m_forge, m_urids.p102_offset_start);
-	lv2_atom_forge_int(&m_forge, (pSample ? pSample->offsetStart() : 0));
-	lv2_atom_forge_key(&m_forge, m_urids.p103_offset_end);
-	lv2_atom_forge_int(&m_forge, (pSample ? pSample->offsetEnd() : 0));
+
+	if (type == 0 || type == m_urids.p101_sample_file) {
+		const char *pszSampleFile = pSample->filename();
+		if (pszSampleFile == nullptr)
+			pszSampleFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p101_sample_file);
+		lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
+	}
+	if (type == 0 || type == m_urids.p102_offset_start) {
+		lv2_atom_forge_key(&m_forge, m_urids.p102_offset_start);
+		lv2_atom_forge_int(&m_forge, (pSample ? pSample->offsetStart() : 0));
+	}
+	if (type == 0 || type == m_urids.p103_offset_end) {
+		lv2_atom_forge_key(&m_forge, m_urids.p103_offset_end);
+		lv2_atom_forge_int(&m_forge, (pSample ? pSample->offsetEnd() : 0));
+	}
+
+	if (type == 0 || type == m_urids.p201_tuning_enabled) {
+		lv2_atom_forge_key(&m_forge, m_urids.p201_tuning_enabled);
+		lv2_atom_forge_bool(&m_forge, drumkv1::isTuningEnabled());
+	}
+	if (type == 0 || type == m_urids.p202_tuning_refPitch) {
+		lv2_atom_forge_key(&m_forge, m_urids.p202_tuning_refPitch);
+		lv2_atom_forge_float(&m_forge, drumkv1::tuningRefPitch());
+	}
+	if (type == 0 || type == m_urids.p203_tuning_refNote) {
+		lv2_atom_forge_key(&m_forge, m_urids.p203_tuning_refNote);
+		lv2_atom_forge_int(&m_forge, drumkv1::tuningRefNote());
+	}
+	if (type == 0 || type == m_urids.p204_tuning_scaleFile) {
+		const char *pszScaleFile = drumkv1::tuningScaleFile();
+		if (pszScaleFile == nullptr)
+			pszScaleFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p204_tuning_scaleFile);
+		lv2_atom_forge_path(&m_forge, pszScaleFile, ::strlen(pszScaleFile) + 1);
+	}
+	if (type == 0 || type == m_urids.p205_tuning_keyMapFile) {
+		const char *pszKeyMapFile = drumkv1::tuningKeyMapFile();
+		if (pszKeyMapFile == nullptr)
+			pszKeyMapFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p205_tuning_keyMapFile);
+		lv2_atom_forge_path(&m_forge, pszKeyMapFile, ::strlen(pszKeyMapFile) + 1);
+	}
 
 	lv2_atom_forge_pop(&m_forge, &body_frame);
 	lv2_atom_forge_pop(&m_forge, &patch_frame);
@@ -718,6 +901,8 @@ static LV2_Handle drumkv1_lv2_instantiate (
 	const LV2_Descriptor *, double sample_rate, const char *,
 	const LV2_Feature *const *host_features )
 {
+	drumkv1_lv2::qapp_instantiate();
+
 	return new drumkv1_lv2(sample_rate, host_features);
 }
 
@@ -760,6 +945,8 @@ static void drumkv1_lv2_cleanup ( LV2_Handle instance )
 	drumkv1_lv2 *pPlugin = static_cast<drumkv1_lv2 *> (instance);
 	if (pPlugin)
 		delete pPlugin;
+
+	drumkv1_lv2::qapp_cleanup();
 }
 
 
@@ -772,7 +959,7 @@ static const LV2_Program_Descriptor *drumkv1_lv2_programs_get_program (
 	if (pPlugin)
 		return pPlugin->get_program(index);
 	else
-		return NULL;
+		return nullptr;
 }
 
 static void drumkv1_lv2_programs_select_program (
@@ -821,7 +1008,7 @@ static const LV2_Worker_Interface drumkv1_lv2_worker_interface =
 {
 	drumkv1_lv2_worker_work,
 	drumkv1_lv2_worker_response,
-	NULL
+	nullptr
 };
 
 
@@ -838,7 +1025,7 @@ static const void *drumkv1_lv2_extension_data ( const char *uri )
 	if (::strcmp(uri, LV2_STATE__interface) == 0)
 		return &drumkv1_lv2_state_interface;
 
-	return NULL;
+	return nullptr;
 }
 
 
@@ -857,7 +1044,7 @@ static const LV2_Descriptor drumkv1_lv2_descriptor =
 
 LV2_SYMBOL_EXPORT const LV2_Descriptor *lv2_descriptor ( uint32_t index )
 {
-	return (index == 0 ? &drumkv1_lv2_descriptor : NULL);
+	return (index == 0 ? &drumkv1_lv2_descriptor : nullptr);
 }
 
 
