@@ -1,7 +1,7 @@
 // drumkv1_lv2.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,7 +24,6 @@
 
 #include "drumkv1.h"
 
-#include "lv2.h"
 #include "lv2/lv2plug.in/ns/ext/urid/urid.h"
 #include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/atom/forge.h"
@@ -40,6 +39,9 @@
 #include <QByteArray>
 #endif
 
+// Forward decls.
+class QApplication;
+
 
 //-------------------------------------------------------------------------
 // drumkv1_lv2 - decl.
@@ -81,16 +83,22 @@ public:
 	bool worker_work(const void *data, uint32_t size);
 	bool worker_response(const void *data, uint32_t size);
 
+	static void qapp_instantiate();
+	static void qapp_cleanup();
+
+	static QApplication *qapp_instance();
+
 protected:
 
 	void updatePreset(bool bDirty);
 	void selectSample(int key);
 	void updateSample();
+	void updateTuning();
 
 	bool state_changed();
 
 #ifdef CONFIG_LV2_PATCH
-	bool patch_put(uint32_t ndelta);
+	bool patch_put(uint32_t ndelta, uint32_t type = 0);
 #endif
 
 private:
@@ -99,13 +107,27 @@ private:
 
 	struct lv2_urids
 	{
+	#if 1//DRUMKV1_LV2_LEGACY
 		LV2_URID gen1_sample;
+		LV2_URID gen1_offset_start;
+		LV2_URID gen1_offset_end;
+	#endif
+		LV2_URID p101_sample_file;
+		LV2_URID p102_offset_start;
+		LV2_URID p103_offset_end;
 		LV2_URID gen1_select;
 		LV2_URID gen1_update;
+		LV2_URID p201_tuning_enabled;
+		LV2_URID p202_tuning_refPitch;
+		LV2_URID p203_tuning_refNote;
+		LV2_URID p204_tuning_scaleFile;
+		LV2_URID p205_tuning_keyMapFile;
+		LV2_URID tun1_update;
 		LV2_URID atom_Blank;
 		LV2_URID atom_Object;
 		LV2_URID atom_Float;
 		LV2_URID atom_Int;
+		LV2_URID atom_Bool;
 		LV2_URID atom_Path;
 		LV2_URID time_Position;
 		LV2_URID time_beatsPerMinute;
@@ -141,6 +163,9 @@ private:
 	LV2_Program_Descriptor m_program;
 	QByteArray m_aProgramName;
 #endif
+
+	static QApplication *g_qapp_instance;
+	static unsigned int  g_qapp_refcount;
 };
 
 
