@@ -1,7 +1,7 @@
 // drumkv1_lv2ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -61,17 +61,13 @@ void drumkv1_lv2ui::write_function (
 // drumkv1_lv2ui - LV2 UI desc.
 //
 
-static QApplication *drumkv1_lv2ui_qapp_instance = NULL;
-static unsigned int  drumkv1_lv2ui_qapp_refcount = 0;
-
-
 static LV2UI_Handle drumkv1_lv2ui_instantiate (
 	const LV2UI_Descriptor *, const char *, const char *,
 	LV2UI_Write_Function write_function,
 	LV2UI_Controller controller, LV2UI_Widget *widget,
 	const LV2_Feature *const *features )
 {
-	drumkv1_lv2 *pSynth = NULL;
+	drumkv1_lv2 *pSynth = nullptr;
 
 	for (int i = 0; features && features[i]; ++i) {
 		if (::strcmp(features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0) {
@@ -80,15 +76,8 @@ static LV2UI_Handle drumkv1_lv2ui_instantiate (
 		}
 	}
 
-	if (pSynth == NULL)
-		return NULL;
-
-	if (qApp == NULL && drumkv1_lv2ui_qapp_instance == NULL) {
-		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, NULL };
-		drumkv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
-	}
-	drumkv1_lv2ui_qapp_refcount++;
+	if (pSynth == nullptr)
+		return nullptr;
 
 	drumkv1widget_lv2 *pWidget
 		= new drumkv1widget_lv2(pSynth, controller, write_function);
@@ -99,13 +88,8 @@ static LV2UI_Handle drumkv1_lv2ui_instantiate (
 static void drumkv1_lv2ui_cleanup ( LV2UI_Handle ui )
 {
 	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
-	if (pWidget) {
+	if (pWidget)
 		delete pWidget;
-		if (--drumkv1_lv2ui_qapp_refcount == 0 && drumkv1_lv2ui_qapp_instance) {
-			delete drumkv1_lv2ui_qapp_instance;
-			drumkv1_lv2ui_qapp_instance = NULL;
-		}
-	}
 }
 
 static void drumkv1_lv2ui_port_event (
@@ -173,6 +157,29 @@ static const LV2UI_Show_Interface drumkv1_lv2ui_show_interface =
 
 #endif	// CONFIG_LV2_UI_IDLE
 
+
+#ifdef CONFIG_LV2_UI_RESIZE
+
+int drumkv1_lv2ui_resize ( LV2UI_Handle ui, int width, int height )
+{
+	drumkv1widget_lv2 *pWidget = static_cast<drumkv1widget_lv2 *> (ui);
+	if (pWidget) {
+		pWidget->resize(width, height);
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
+static const LV2UI_Resize drumkv1_lv2ui_resize_interface =
+{
+	nullptr, // handle: host should use its own when calling ui_resize().
+	drumkv1_lv2ui_resize
+};
+
+#endif	// CONFIG_LV2_UI_RESIZE
+
+
 static const void *drumkv1_lv2ui_extension_data ( const char *uri )
 {
 #ifdef CONFIG_LV2_UI_IDLE
@@ -185,7 +192,12 @@ static const void *drumkv1_lv2ui_extension_data ( const char *uri )
 		return (void *) &drumkv1_lv2ui_show_interface;
 	else
 #endif
-	return NULL;
+#ifdef CONFIG_LV2_UI_RESIZE
+	if (::strcmp(uri, LV2_UI__resize) == 0)
+		return (void *) &drumkv1_lv2ui_resize_interface;
+	else
+#endif
+	return nullptr;
 }
 
 
@@ -198,8 +210,8 @@ static LV2UI_Handle drumkv1_lv2ui_x11_instantiate (
 	const LV2_Feature *const *ui_features )
 {
 	WId winid, parent = 0;
-	LV2UI_Resize *resize = NULL;
-	drumkv1_lv2 *pDrumk = NULL;
+	LV2UI_Resize *resize = nullptr;
+	drumkv1_lv2 *pDrumk = nullptr;
 
 	for (int i = 0; ui_features[i]; ++i) {
 		if (::strcmp(ui_features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0)
@@ -212,17 +224,10 @@ static LV2UI_Handle drumkv1_lv2ui_x11_instantiate (
 			resize = (LV2UI_Resize *) ui_features[i]->data;
 	}
 
-	if (pDrumk == NULL)
-		return NULL;
+	if (pDrumk == nullptr)
+		return nullptr;
 	if (!parent)
-		return NULL;
-
-	if (qApp == NULL && drumkv1_lv2ui_qapp_instance == NULL) {
-		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, NULL };
-		drumkv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
-	}
-	drumkv1_lv2ui_qapp_refcount++;
+		return nullptr;
 
 	drumkv1widget_lv2 *pWidget
 		= new drumkv1widget_lv2(pDrumk, controller, write_function);
@@ -284,8 +289,8 @@ static LV2UI_Handle drumkv1_lv2ui_external_instantiate (
 	LV2UI_Controller controller, LV2UI_Widget *widget,
 	const LV2_Feature *const *ui_features )
 {
-	drumkv1_lv2 *pSynth = NULL;
-	LV2_External_UI_Host *external_host = NULL;
+	drumkv1_lv2 *pSynth = nullptr;
+	LV2_External_UI_Host *external_host = nullptr;
 
 	for (int i = 0; ui_features[i] && !external_host; ++i) {
 		if (::strcmp(ui_features[i]->URI, LV2_INSTANCE_ACCESS_URI) == 0)
@@ -297,13 +302,6 @@ static LV2UI_Handle drumkv1_lv2ui_external_instantiate (
 		}
 	}
 
-	if (qApp == NULL && drumkv1_lv2ui_qapp_instance == NULL) {
-		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, NULL };
-		drumkv1_lv2ui_qapp_instance = new QApplication(s_argc, (char **) s_argv);
-	}
-	drumkv1_lv2ui_qapp_refcount++;
-
 	drumkv1_lv2ui_external_widget *pExtWidget = new drumkv1_lv2ui_external_widget;
 	pExtWidget->external.run  = drumkv1_lv2ui_external_run;
 	pExtWidget->external.show = drumkv1_lv2ui_external_show;
@@ -323,10 +321,6 @@ static void drumkv1_lv2ui_external_cleanup ( LV2UI_Handle ui )
 		if (pExtWidget->widget)
 			delete pExtWidget->widget;
 		delete pExtWidget;
-		if (--drumkv1_lv2ui_qapp_refcount == 0 && drumkv1_lv2ui_qapp_instance) {
-			delete drumkv1_lv2ui_qapp_instance;
-			drumkv1_lv2ui_qapp_instance = NULL;
-		}
 	}
 }
 
@@ -342,7 +336,7 @@ static void drumkv1_lv2ui_external_port_event (
 
 static const void *drumkv1_lv2ui_external_extension_data ( const char * )
 {
-	return NULL;
+	return nullptr;
 }
 
 #endif	// CONFIG_LV2_UI_EXTERNAL
@@ -395,7 +389,7 @@ LV2_SYMBOL_EXPORT const LV2UI_Descriptor *lv2ui_descriptor ( uint32_t index )
 		return &drumkv1_lv2ui_external_descriptor;
 	else
 #endif
-	return NULL;
+	return nullptr;
 }
 
 
