@@ -1,7 +1,7 @@
 // drumkv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -51,10 +51,13 @@ struct ParamInfo {
 	{ "GEN1_SAMPLE",   PARAM_INT,    36.0f,   0.0f, 127.0f }, // GEN1 Sample
 	{ "GEN1_REVERSE",  PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Reverse
 	{ "GEN1_OFFSET",   PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // GEN1 Offset
+	{ "GEN1_OFFSET_1", PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // GEN1 Offset Start
+	{ "GEN1_OFFSET_2", PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // GEN1 Offset End
 	{ "GEN1_GROUP",    PARAM_FLOAT,   0.0f,   0.0f, 128.0f }, // GEN1 Group
 	{ "GEN1_COARSE",   PARAM_FLOAT,   0.0f,  -4.0f,   4.0f }, // GEN1 Coarse
 	{ "GEN1_FINE",     PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // GEN1 Fine
 	{ "GEN1_ENVTIME",  PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // GEN1 Env.Time
+	{ "DCF1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCF1 Enabled
 	{ "DCF1_CUTOFF",   PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // DCF1 Cutoff
 	{ "DCF1_RESO",     PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCF1 Resonance
 	{ "DCF1_TYPE",     PARAM_INT,     0.0f,   0.0f,   3.0f }, // DCF1 Type
@@ -64,11 +67,11 @@ struct ParamInfo {
 	{ "DCF1_DECAY1",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Decay 1
 	{ "DCF1_LEVEL2",   PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DCF1 Level 2
 	{ "DCF1_DECAY2",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Decay 2
+	{ "LFO1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // LFO1 Enabled
 	{ "LFO1_SHAPE",    PARAM_INT,     1.0f,   0.0f,   4.0f }, // LFO1 Wave Shape
 	{ "LFO1_WIDTH",    PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // LFO1 Wave Width
 	{ "LFO1_BPM",      PARAM_FLOAT, 180.0f,   0.0f, 360.0f }, // LFO1 BPM
 	{ "LFO1_RATE",     PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // LFO1 Rate
-	{ "LFO1_SYNC",     PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // LFO1 Sync
 	{ "LFO1_SWEEP",    PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // LFO1 Sweep
 	{ "LFO1_PITCH",    PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // LFO1 Pitch
 	{ "LFO1_CUTOFF",   PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // LFO1 Cutoff
@@ -79,6 +82,7 @@ struct ParamInfo {
 	{ "LFO1_DECAY1",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // LFO1 Decay 1
 	{ "LFO1_LEVEL2",   PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // LFO1 Level 2
 	{ "LFO1_DECAY2",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // LFO1 Decay 2
+	{ "DCA1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCA1 Enabled
 	{ "DCA1_VOLUME",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCA1 Volume
 	{ "DCA1_ATTACK",   PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCA1 Attack
 	{ "DCA1_DECAY1",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCA1 Decay1
@@ -185,12 +189,35 @@ QString drumkv1_param::map_path::abstractPath (
 }
 
 
+float drumkv1_param::paramScale ( drumkv1::ParamIndex index, float fValue )
+{
+	const ParamInfo& param = drumkv1_params[index];
+
+	if (param.type == PARAM_BOOL)
+		return (fValue > 0.5f ? 1.0f : 0.0f);
+
+	const float fScale = (fValue - param.min) / (param.max - param.min);
+
+	if (param.type == PARAM_INT)
+		return ::rintf(fScale);
+	else
+		return fScale;
+}
+
+
+bool drumkv1_param::paramFloat ( drumkv1::ParamIndex index )
+{
+	return (drumkv1_params[index].type == PARAM_FLOAT);
+}
+
+
+
 // Element serialization methods.
 void drumkv1_param::loadElements (
 	drumkv1 *pDrumk, const QDomElement& eElements,
 	const drumkv1_param::map_path& mapPath )
 {
-	if (pDrumk == NULL)
+	if (pDrumk == nullptr)
 		return;
 
 	pDrumk->clearElements();
@@ -228,11 +255,12 @@ void drumkv1_param::loadElements (
 						= eChild.attribute("offset-start").toULong();
 					const uint32_t iOffsetEnd
 						= eChild.attribute("offset-end").toULong();
-					QFileInfo fi(eChild.text());
-					if (fi.isSymLink())
-						fi.setFile(fi.symLinkTarget());
-					element->setSampleFile(
-						mapPath.absolutePath(fi.filePath()).toUtf8().constData());
+					const QString& sSampleFile
+						= eChild.text();
+					const QByteArray aSampleFile
+						= mapPath.absolutePath(
+							drumkv1_param::loadFilename(sSampleFile)).toUtf8();
+					element->setSampleFile(aSampleFile.constData());
 					element->setOffsetRange(iOffsetStart, iOffsetEnd);
 				}
 				else
@@ -267,28 +295,16 @@ void drumkv1_param::saveElements (
 	drumkv1 *pDrumk, QDomDocument& doc, QDomElement& eElements,
 	const drumkv1_param::map_path& mapPath, bool bSymLink )
 {
-	if (pDrumk == NULL)
+	if (pDrumk == nullptr)
 		return;
 
 	for (int note = 0; note < 128; ++note) {
 		drumkv1_element *element = pDrumk->element(note);
-		if (element == NULL)
+		if (element == nullptr)
 			continue;
 		const char *pszSampleFile = element->sampleFile();
-		if (pszSampleFile == NULL)
+		if (pszSampleFile == nullptr)
 			continue;
-		QFileInfo fi(QString::fromUtf8(pszSampleFile));
-		if (bSymLink && fi.absolutePath() != QDir::current().absolutePath()) {
-			const QString& sPath = fi.absoluteFilePath();
-			const QString& sName = fi.baseName();
-			const QString& sExt  = fi.completeSuffix();
-			const QString& sLink = sName
-				+ '-' + QString::number(qHash(sPath), 16)
-				+ '.' + sExt;
-			QFile(sPath).link(sLink);
-			fi.setFile(QDir::current(), sLink);
-		}
-		else if (fi.isSymLink()) fi.setFile(fi.symLinkTarget());
 		QDomElement eElement = doc.createElement("element");
 		eElement.setAttribute("index", QString::number(note));
 	//	eElement.setAttribute("name", noteName(note));
@@ -297,8 +313,9 @@ void drumkv1_param::saveElements (
 		eSample.setAttribute("name", "GEN1_SAMPLE");
 		eSample.setAttribute("offset-start", element->offsetStart());
 		eSample.setAttribute("offset-end", element->offsetEnd());
-		eSample.appendChild(doc.createTextNode(
-			mapPath.abstractPath(fi.absoluteFilePath())));
+		eSample.appendChild(doc.createTextNode(mapPath.abstractPath(
+			drumkv1_param::saveFilename(
+				QString::fromUtf8(pszSampleFile), bSymLink))));
 		eElement.appendChild(eSample);
 		QDomElement eParams = doc.createElement("params");
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
@@ -316,34 +333,11 @@ void drumkv1_param::saveElements (
 }
 
 
-float drumkv1_param::paramScale ( drumkv1::ParamIndex index, float fValue )
-{
-	const ParamInfo& param = drumkv1_params[index];
-
-	if (param.type == PARAM_BOOL)
-		return (fValue > 0.5f ? 1.0f : 0.0f);
-
-	const float fScale = (fValue - param.min) / (param.max - param.min);
-
-	if (param.type == PARAM_INT)
-		return ::rintf(fScale);
-	else
-		return fScale;
-}
-
-
-bool drumkv1_param::paramFloat ( drumkv1::ParamIndex index )
-{
-	return (drumkv1_params[index].type == PARAM_FLOAT);
-}
-
-
-
 // Preset serialization methods.
 bool drumkv1_param::loadPreset (
 	drumkv1 *pDrumk, const QString& sFilename )
 {
-	if (pDrumk == NULL)
+	if (pDrumk == nullptr)
 		return false;
 
 	QFileInfo fi(sFilename);
@@ -364,6 +358,11 @@ bool drumkv1_param::loadPreset (
 	if (!file.open(QIODevice::ReadOnly))
 		return false;
 
+	const bool running = pDrumk->running(false);
+
+	pDrumk->setTuningEnabled(false);
+	pDrumk->reset();
+
 	static QHash<QString, drumkv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
 		for (uint32_t i = drumkv1::NUM_ELEMENT_PARAMS; i < drumkv1::NUM_PARAMS; ++i) {
@@ -386,10 +385,6 @@ bool drumkv1_param::loadPreset (
 				QDomElement eChild = nChild.toElement();
 				if (eChild.isNull())
 					continue;
-				if (eChild.tagName() == "elements") {
-					drumkv1_param::loadElements(pDrumk, eChild);
-				}
-				else
 				if (eChild.tagName() == "params") {
 					for (QDomNode nParam = eChild.firstChild();
 							!nParam.isNull();
@@ -412,13 +407,23 @@ bool drumkv1_param::loadPreset (
 						}
 					}
 				}
+				else
+				if (eChild.tagName() == "elements") {
+					drumkv1_param::loadElements(pDrumk, eChild);
+				}
+				else
+				if (eChild.tagName() == "tuning") {
+					drumkv1_param::loadTuning(pDrumk, eChild);
+				}
 			}
 		}
 	}
 
 	file.close();
 
+	pDrumk->stabilize();
 	pDrumk->reset();
+	pDrumk->running(running);
 
 	QDir::setCurrent(currentDir.absolutePath());
 
@@ -429,9 +434,11 @@ bool drumkv1_param::loadPreset (
 bool drumkv1_param::savePreset (
 	drumkv1 *pDrumk, const QString& sFilename, bool bSymLink )
 {
-	if (pDrumk == NULL)
+	if (pDrumk == nullptr)
 		return false;
 
+	pDrumk->stabilize();
+
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
@@ -458,6 +465,12 @@ bool drumkv1_param::savePreset (
 	ePreset.appendChild(eParams);
 	doc.appendChild(ePreset);
 
+	if (pDrumk->isTuningEnabled()) {
+		QDomElement eTuning = doc.createElement("tuning");
+		drumkv1_param::saveTuning(pDrumk, doc, eTuning, bSymLink);
+		ePreset.appendChild(eTuning);
+	}
+
 	QFile file(fi.filePath());
 	if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate))
 		return false;
@@ -471,4 +484,126 @@ bool drumkv1_param::savePreset (
 }
 
 
+// Tuning serialization methods.
+void drumkv1_param::loadTuning (
+	drumkv1 *pDrumk, const QDomElement& eTuning )
+{
+	if (pDrumk == nullptr)
+		return;
+
+	pDrumk->setTuningEnabled(eTuning.attribute("enabled").toInt() > 0);
+
+	for (QDomNode nChild = eTuning.firstChild();
+			!nChild.isNull();
+				nChild = nChild.nextSibling()) {
+		QDomElement eChild = nChild.toElement();
+		if (eChild.isNull())
+			continue;
+		if (eChild.tagName() == "enabled") {
+			pDrumk->setTuningEnabled(eChild.text().toInt() > 0);
+		}
+		if (eChild.tagName() == "ref-pitch") {
+			pDrumk->setTuningRefPitch(eChild.text().toFloat());
+		}
+		else
+		if (eChild.tagName() == "ref-note") {
+			pDrumk->setTuningRefNote(eChild.text().toInt());
+		}
+		else
+		if (eChild.tagName() == "scale-file") {
+			const QString& sScaleFile
+				= eChild.text();
+			const QByteArray aScaleFile
+				= drumkv1_param::loadFilename(sScaleFile).toUtf8();
+			pDrumk->setTuningScaleFile(aScaleFile.constData());
+		}
+		else
+		if (eChild.tagName() == "keymap-file") {
+			const QString& sKeyMapFile
+				= eChild.text();
+			const QByteArray aKeyMapFile
+				= drumkv1_param::loadFilename(sKeyMapFile).toUtf8();
+			pDrumk->setTuningScaleFile(aKeyMapFile.constData());
+		}
+	}
+
+	// Consolidate tuning state...
+	pDrumk->resetTuning();
+}
+
+
+void drumkv1_param::saveTuning (
+	drumkv1 *pDrumk, QDomDocument& doc, QDomElement& eTuning, bool bSymLink )
+{
+	if (pDrumk == nullptr)
+		return;
+
+	eTuning.setAttribute("enabled", int(pDrumk->isTuningEnabled()));
+
+	QDomElement eRefPitch = doc.createElement("ref-pitch");
+	eRefPitch.appendChild(doc.createTextNode(
+		QString::number(pDrumk->tuningRefPitch())));
+	eTuning.appendChild(eRefPitch);
+
+	QDomElement eRefNote = doc.createElement("ref-note");
+	eRefNote.appendChild(doc.createTextNode(
+		QString::number(pDrumk->tuningRefNote())));
+	eTuning.appendChild(eRefNote);
+
+	const char *pszScaleFile = pDrumk->tuningScaleFile();
+	if (pszScaleFile) {
+		const QString& sScaleFile
+			= QString::fromUtf8(pszScaleFile);
+		if (!sScaleFile.isEmpty()) {
+			QDomElement eScaleFile = doc.createElement("scale-file");
+			eScaleFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					drumkv1_param::saveFilename(sScaleFile, bSymLink))));
+			eTuning.appendChild(eScaleFile);
+		}
+	}
+
+	const char *pszKeyMapFile = pDrumk->tuningKeyMapFile();
+	if (pszKeyMapFile) {
+		const QString& sKeyMapFile
+			= QString::fromUtf8(pszKeyMapFile);
+		if (!sKeyMapFile.isEmpty()) {
+			QDomElement eKeyMapFile = doc.createElement("keymap-file");
+			eKeyMapFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					drumkv1_param::saveFilename(sKeyMapFile, bSymLink))));
+			eTuning.appendChild(eKeyMapFile);
+		}
+	}
+}
+
+
+// Load/save and convert canonical/absolute filename helpers.
+QString drumkv1_param::loadFilename ( const QString& sFilename )
+{
+	QFileInfo fi(sFilename);
+	if (fi.isSymLink())
+		fi.setFile(fi.symLinkTarget());
+	return fi.filePath();
+}
+
+
+QString drumkv1_param::saveFilename ( const QString& sFilename, bool bSymLink )
+{
+	QFileInfo fi(sFilename);
+	if (bSymLink && fi.absolutePath() != QDir::current().absolutePath()) {
+		const QString& sPath = fi.absoluteFilePath();
+		const QString& sName = fi.baseName();
+		const QString& sExt  = fi.completeSuffix();
+		const QString& sLink = sName
+			+ '-' + QString::number(qHash(sPath), 16)
+			+ '.' + sExt;
+		QFile(sPath).link(sLink);
+		fi.setFile(QDir::current(), sLink);
+	}
+	else if (fi.isSymLink()) fi.setFile(fi.symLinkTarget());
+	return fi.absoluteFilePath();
+}
+
+
 // end of drumkv1_param.cpp
