@@ -1,7 +1,7 @@
 // drumkv1_param.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -46,6 +46,13 @@ namespace drumkv1_param
 		virtual QString abstractPath(const QString& sAbsolutePath) const;
 	};
 
+	// Preset serialization methods.
+	bool loadPreset(drumkv1 *pDrumk,
+		const QString& sFilename);
+	bool savePreset(drumkv1 *pDrumk,
+		const QString& sFilename,
+		bool bSymLink = false);
+
 	// Element serialization methods.
 	void loadElements(drumkv1 *pDrumk,
 		const QDomElement& eElements,
@@ -55,11 +62,11 @@ namespace drumkv1_param
 		const map_path& mapPath = map_path(),
 		bool bSymLink = false);
 
-	// Preset serialization methods.
-	bool loadPreset(drumkv1 *pDrumk,
-		const QString& sFilename);
-	bool savePreset(drumkv1 *pDrumk,
-		const QString& sFilename,
+	// Tuning serialization methods.
+	void loadTuning(drumkv1 *pDrumk,
+		const QDomElement& eTuning);
+	void saveTuning(drumkv1 *pDrumk,
+		QDomDocument& doc, QDomElement& eTuning,
 		bool bSymLink = false);
 
 	// Default parameter name/value helpers.
@@ -69,6 +76,10 @@ namespace drumkv1_param
 	float paramValue(drumkv1::ParamIndex index, float fScale);
 	float paramScale(drumkv1::ParamIndex index, float fValue);
 	bool paramFloat(drumkv1::ParamIndex index);
+
+	// Load/save and convert canonical/absolute filename helpers.
+	QString loadFilename(const QString& sFilename);
+	QString saveFilename(const QString& sFilename, bool bSymLink);
 };
 
 
