@@ -1,7 +1,7 @@
 // drumkv1_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,9 +32,9 @@
 
 // ctor.
 drumkv1_sample::drumkv1_sample ( float srate )
-	: m_srate(srate), m_filename(NULL), m_nchannels(0),
+	: m_srate(srate), m_filename(nullptr), m_nchannels(0),
 		m_rate0(0.0f), m_freq0(1.0f), m_ratio(0.0f),
-		m_nframes(0), m_pframes(NULL), m_reverse(false),
+		m_nframes(0), m_pframes(nullptr), m_reverse(false),
 		m_offset(false), m_offset_start(0), m_offset_end(0),
 		m_offset_phase0(0.0f), m_offset_end2(0)
 {
@@ -51,7 +51,7 @@ drumkv1_sample::~drumkv1_sample (void)
 // init.
 bool drumkv1_sample::open ( const char *filename, float freq0 )
 {
-	if (filename == NULL)
+	if (filename == nullptr)
 		return false;
 
 	close();
@@ -62,7 +62,7 @@ bool drumkv1_sample::open ( const char *filename, float freq0 )
 	::memset(&info, 0, sizeof(info));
 	
 	SNDFILE *file = ::sf_open(m_filename, SFM_READ, &info);
-	if (file == NULL)
+	if (file == nullptr)
 		return false;
 
 	m_nchannels = info.channels;
@@ -121,7 +121,7 @@ bool drumkv1_sample::open ( const char *filename, float freq0 )
 
 	reset(freq0);
 
-	setOffset(m_offset);
+	updateOffset();
 	return true;
 }
 
@@ -132,7 +132,7 @@ void drumkv1_sample::close (void)
 		for (uint16_t k = 0; k < m_nchannels; ++k)
 			delete [] m_pframes[k];
 		delete [] m_pframes;
-		m_pframes = NULL;
+		m_pframes = nullptr;
 	}
 
 	m_nframes   = 0;
@@ -145,7 +145,7 @@ void drumkv1_sample::close (void)
 
 	if (m_filename) {
 		::free(m_filename);
-		m_filename = NULL;
+		m_filename = nullptr;
 	}
 }
 
@@ -181,28 +181,37 @@ void drumkv1_sample::setOffsetRange ( uint32_t start, uint32_t end )
 	if (start < end) {
 		m_offset_start = start;
 		m_offset_end = end;
-		m_offset_phase0 = float(zero_crossing(start, NULL));
-		m_offset_end2 = zero_crossing(end, NULL);
 	} else {
 		m_offset_start = 0;
 		m_offset_end = m_nframes;
+	}
+
+	if (m_offset && m_offset_start < m_offset_end) {
+		m_offset_phase0 = float(zero_crossing(m_offset_start, nullptr));
+		m_offset_end2 = zero_crossing(m_offset_end, nullptr);
+	} else {
 		m_offset_phase0 = 0.0f;
 		m_offset_end2 = m_nframes;
 	}
 }
 
 
-// zero-crossing aliasing (single channel).
-uint32_t drumkv1_sample::zero_crossing_k (
-	uint32_t i, uint16_t k, int *slope ) const
+// offset updater.
+void drumkv1_sample::updateOffset (void)
+{
+	setOffsetRange(m_offset_start, m_offset_end);
+}
+
+
+// zero-crossing aliasing (all channels).
+uint32_t drumkv1_sample::zero_crossing ( uint32_t i, int *slope ) const
 {
-	const float *frames = m_pframes[k];
 	const int s0 = (slope ? *slope : 0);
 
 	if (i > 0) --i;
-	float v0 = frames[i];
+	float v0 = zero_crossing_k(i);
 	for (++i; i < m_nframes; ++i) {
-		const float v1 = frames[i];
+		const float v1 = zero_crossing_k(i);
 		if ((0 >= s0 && v0 >= 0.0f && 0.0f >= v1) ||
 			(s0 >= 0 && v1 >= 0.0f && 0.0f >= v0)) {
 			if (slope && s0 == 0) *slope = (v1 < v0 ? -1 : +1);
@@ -216,12 +225,12 @@ uint32_t drumkv1_sample::zero_crossing_k (
 
 
 // zero-crossing aliasing (median).
-uint32_t drumkv1_sample::zero_crossing ( uint32_t i, int *slope ) const
+float drumkv1_sample::zero_crossing_k ( uint32_t i ) const
 {
-	uint32_t sum = 0;
+	float sum = 0.0f;
 	for (uint16_t k = 0; k < m_nchannels; ++k)
-		sum += zero_crossing_k(i, k, slope);
-	return (sum / m_nchannels);
+		sum += m_pframes[k][i];
+	return (sum / float(m_nchannels));
 }
 
 
