@@ -1,7 +1,7 @@
 // drumkv1_sched.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -63,6 +63,27 @@ public:
 	// signal broadcast (static).
 	static void sync_notify(drumkv1 *pDrumk, Type stype, int sid);
 
+	// Notifier - Worker/schedule proxy decl.
+	//
+	class Notifier
+	{
+	public:
+
+		// ctor.
+		Notifier(drumkv1 *pDrumk);
+
+		// dtor.
+		virtual ~Notifier();
+
+		// signal notifier.
+		virtual void notify(drumkv1_sched::Type stype, int sid) const = 0;
+
+	private:
+
+		// instance variables.
+		drumkv1 *m_pDrumk;
+	};
+
 private:
 
 	// instance variables.
@@ -83,30 +104,6 @@ private:
 };
 
 
-//-------------------------------------------------------------------------
-// drumkv1_sched_notifier - worker/schedule proxy decl.
-//
-
-class drumkv1_sched_notifier
-{
-public:
-
-	// ctor.
-	drumkv1_sched_notifier(drumkv1 *pDrumk);
-
-	// dtor.
-	~drumkv1_sched_notifier();
-
-	// signal notifier.
-	virtual void notify(drumkv1_sched::Type stype, int sid) const = 0;
-
-private:
-
-	// instance variables.
-	drumkv1 *m_pDrumk;
-};
-
-
 #endif	// __drumkv1_sched_h
 
 // end of drumkv1_sched.h
