@@ -1,7 +1,7 @@
 // drumkv1_ui.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -204,18 +204,87 @@ void drumkv1_ui::directNoteOn ( int note, int vel )
 }
 
 
-void drumkv1_ui::updateTuning (void)
+void drumkv1_ui::setTuningEnabled ( bool enabled )
 {
-	m_pDrumk->updateTuning();
+	m_pDrumk->setTuningEnabled(enabled);
+}
+
+bool drumkv1_ui::isTuningEnabled (void) const
+{
+	return m_pDrumk->isTuningEnabled();
+}
+
+
+void drumkv1_ui::setTuningRefPitch ( float refPitch )
+{
+	m_pDrumk->setTuningRefPitch(refPitch);
+}
+
+float drumkv1_ui::tuningRefPitch (void) const
+{
+	return m_pDrumk->tuningRefPitch();
+}
+
+
+void drumkv1_ui::setTuningRefNote ( int refNote )
+{
+	m_pDrumk->setTuningRefNote(refNote);
+}
+
+int drumkv1_ui::tuningRefNote (void) const
+{
+	return m_pDrumk->tuningRefNote();
+}
+
+
+void drumkv1_ui::setTuningScaleFile ( const char *pszScaleFile )
+{
+	m_pDrumk->setTuningScaleFile(pszScaleFile);
+}
+
+const char *drumkv1_ui::tuningScaleFile (void) const
+{
+	return m_pDrumk->tuningScaleFile();
+}
+
+
+void drumkv1_ui::setTuningKeyMapFile ( const char *pszKeyMapFile )
+{
+	m_pDrumk->setTuningKeyMapFile(pszKeyMapFile);
+}
+
+const char *drumkv1_ui::tuningKeyMapFile (void) const
+{
+	return m_pDrumk->tuningKeyMapFile();
+}
+
+
+void drumkv1_ui::resetTuning (void)
+{
+	m_pDrumk->resetTuning();
 }
 
 
 // MIDI note/octave name helper (static).
 QString drumkv1_ui::noteName ( int note )
 {
-	static const char *notes[] =
-		{ "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
-	return QString("%1 %2").arg(notes[note % 12]).arg((note / 12) - 1);
+	static const char *s_notes[] =
+	{
+		QT_TR_NOOP("C"),
+		QT_TR_NOOP("C#/Db"),
+		QT_TR_NOOP("D"),
+		QT_TR_NOOP("D#/Eb"),
+		QT_TR_NOOP("E"),
+		QT_TR_NOOP("F"),
+		QT_TR_NOOP("F#/Gb"),
+		QT_TR_NOOP("G"),
+		QT_TR_NOOP("G#/Ab"),
+		QT_TR_NOOP("A"),
+		QT_TR_NOOP("A#/Bb"),
+		QT_TR_NOOP("B")
+	};
+
+	return QString("%1 %2").arg(s_notes[note % 12]).arg((note / 12) - 1);
 }
 
 
