@@ -1,7 +1,7 @@
 // drumkv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 #include "drumkv1widget_config.h"
 #include "drumkv1widget_control.h"
 
+#include "drumkv1widget_keybd.h"
+
 #include "drumkv1_controls.h"
 #include "drumkv1_programs.h"
 
@@ -40,6 +42,10 @@
 #include <QShowEvent>
 #include <QHideEvent>
 
+#include <math.h>
+
+#include <random>
+
 
 //-------------------------------------------------------------------------
 // drumkv1widget - impl.
@@ -51,7 +57,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 {
 	Q_INIT_RESOURCE(drumkv1);
 
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	// HACK: Dark themes grayed/disabled color group fix...
 	QPalette pal;
 	if (pal.base().color().value() < 0x7f) {
@@ -73,7 +79,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = NULL;
+	m_sched_notifier = nullptr;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i)
@@ -161,13 +167,8 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 #if 0
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
 	m_ui.Gen1OffsetKnob->insertItems(0, states);
-
-	m_ui.Lfo1SyncKnob->insertItems(0, states);
-
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
-#else
-	m_ui.Lfo1SyncKnob->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
 #endif
 	// Special values
 	const QString& sOff = states.first();
@@ -246,6 +247,7 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(drumkv1::GEN1_ENVTIME, m_ui.Gen1EnvTimeKnob);
 
 	// DCF1
+	setParamKnob(drumkv1::DCF1_ENABLED,  m_ui.Dcf1GroupBox->param());
 	setParamKnob(drumkv1::DCF1_CUTOFF,   m_ui.Dcf1CutoffKnob);
 	setParamKnob(drumkv1::DCF1_RESO,     m_ui.Dcf1ResoKnob);
 	setParamKnob(drumkv1::DCF1_TYPE,     m_ui.Dcf1TypeKnob);
@@ -306,11 +308,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dcf1Env, SLOT(setDecay2(float)));
 
 	// LFO1
+	setParamKnob(drumkv1::LFO1_ENABLED, m_ui.Lfo1GroupBox->param());
 	setParamKnob(drumkv1::LFO1_SHAPE,   m_ui.Lfo1ShapeKnob);
 	setParamKnob(drumkv1::LFO1_WIDTH,   m_ui.Lfo1WidthKnob);
 	setParamKnob(drumkv1::LFO1_BPM,     m_ui.Lfo1BpmKnob);
 	setParamKnob(drumkv1::LFO1_RATE,    m_ui.Lfo1RateKnob);
-	setParamKnob(drumkv1::LFO1_SYNC,    m_ui.Lfo1SyncKnob);
 	setParamKnob(drumkv1::LFO1_PANNING, m_ui.Lfo1PanningKnob);
 	setParamKnob(drumkv1::LFO1_VOLUME,  m_ui.Lfo1VolumeKnob);
 	setParamKnob(drumkv1::LFO1_CUTOFF,  m_ui.Lfo1CutoffKnob);
@@ -364,11 +366,12 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Lfo1Env, SLOT(setDecay2(float)));
 
 	// DCA1
-	setParamKnob(drumkv1::DCA1_VOLUME, m_ui.Dca1VolumeKnob);
-	setParamKnob(drumkv1::DCA1_ATTACK, m_ui.Dca1AttackKnob);
-	setParamKnob(drumkv1::DCA1_DECAY1, m_ui.Dca1Decay1Knob);
-	setParamKnob(drumkv1::DCA1_LEVEL2, m_ui.Dca1Level2Knob);
-	setParamKnob(drumkv1::DCA1_DECAY2, m_ui.Dca1Decay2Knob);
+	setParamKnob(drumkv1::DCA1_ENABLED, m_ui.Dca1GroupBox->param());
+	setParamKnob(drumkv1::DCA1_VOLUME,  m_ui.Dca1VolumeKnob);
+	setParamKnob(drumkv1::DCA1_ATTACK,  m_ui.Dca1AttackKnob);
+	setParamKnob(drumkv1::DCA1_DECAY1,  m_ui.Dca1Decay1Knob);
+	setParamKnob(drumkv1::DCA1_LEVEL2,  m_ui.Dca1Level2Knob);
+	setParamKnob(drumkv1::DCA1_DECAY2,  m_ui.Dca1Decay2Knob);
 
 	QObject::connect(
 		m_ui.Dca1Env, SIGNAL(attackChanged(float)),
@@ -512,6 +515,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(customContextMenuRequested(const QPoint&)),
 		SLOT(spinboxContextMenu(const QPoint&)));
 
+	// Randomize params...
+	QObject::connect(m_ui.RandomParamsButton,
+		SIGNAL(clicked()),
+		SLOT(randomParams()));
+
 	// Swap params A/B
 	QObject::connect(m_ui.SwapParamsAButton,
 		SIGNAL(toggled(bool)),
@@ -524,6 +532,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	// Direct status-bar keyboard input
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteOnClicked(int, int)),
+		SLOT(directNoteOn(int, int)));
+
 	// Menu actions
 	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
@@ -574,7 +587,7 @@ void drumkv1widget::openSchedNotifier (void)
 		return;
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	m_sched_notifier = new drumkv1widget_sched(pDrumkUi->instance(), this);
@@ -591,7 +604,7 @@ void drumkv1widget::closeSchedNotifier (void)
 {
 	if (m_sched_notifier) {
 		delete m_sched_notifier;
-		m_sched_notifier = NULL;
+		m_sched_notifier = nullptr;
 	}
 
 	drumkv1_ui *pDrumkUi = ui_instance();
@@ -638,7 +651,7 @@ void drumkv1widget::setParamKnob ( drumkv1::ParamIndex index, drumkv1widget_para
 
 drumkv1widget_param *drumkv1widget::paramKnob ( drumkv1::ParamIndex index ) const
 {
-	return m_paramKnobs.value(index, NULL);
+	return m_paramKnobs.value(index, nullptr);
 }
 
 
@@ -707,25 +720,47 @@ void drumkv1widget::updateParamEx (
 	drumkv1::ParamIndex index, float fValue, bool bIter )
 {
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	++m_iUpdate;
 
 	switch (index) {
-	case drumkv1::GEN1_REVERSE: {
+	case drumkv1::GEN1_REVERSE:
 		pDrumkUi->setReverse(bool(fValue > 0.0f));
 		if (!bIter) updateSample(pDrumkUi->sample());
 		break;
-	}
-	case drumkv1::GEN1_OFFSET: {
+	case drumkv1::GEN1_OFFSET:
 		pDrumkUi->setOffset(bool(fValue > 0.0f));
 		if (!bIter) updateOffset(pDrumkUi->sample());
 		break;
-	}
+	case drumkv1::DCF1_ENABLED:
+		if (m_ui.Lfo1GroupBox->isChecked()) {
+			const bool bDcf1Enabled = (fValue > 0.5f);
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
+		break;
+	case drumkv1::LFO1_ENABLED:
+		if (fValue > 0.5f) {
+			const bool bDcf1Enabled = m_ui.Dcf1GroupBox->isChecked();
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
+		break;
 	case drumkv1::DCF1_SLOPE:
-		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
-		// Fall thru...
+		if (m_ui.Dcf1GroupBox->isChecked())
+			m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+		break;
+	case drumkv1::LFO1_SHAPE:
+		m_ui.Lfo1Wave->setWaveShape(fValue);
+		break;
+	case drumkv1::DEF1_VELOCITY: {
+		const int vel = int(79.375f * fValue + 47.625f) & 0x7f;
+		m_ui.Elements->setDirectNoteOnVelocity(vel);
+		m_ui.StatusBar->keybd()->setVelocity(vel);
+		break;
+	}
 	default:
 		break;
 	}
@@ -758,7 +793,7 @@ void drumkv1widget::updateSchedParam ( drumkv1::ParamIndex index, float fValue )
 void drumkv1widget::resetParams (void)
 {
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	pDrumkUi->reset();
@@ -785,6 +820,70 @@ void drumkv1widget::resetParams (void)
 }
 
 
+// Randomize params (partial).
+void drumkv1widget::randomParams (void)
+{
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi == nullptr)
+		return;
+
+	float p = 1.0f;
+
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig)
+		p = 0.01f * pConfig->fRandomizePercent;
+
+	if (QMessageBox::warning(this,
+		tr("Warning"),
+		tr("About to randomize current parameter values:\n\n"
+		"-/+ %1%.\n\n"
+		"Are you sure?").arg(100.0f * p),
+		QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
+		return;
+
+	std::default_random_engine re(::time(nullptr));
+
+	for (uint32_t i = 0; i < drumkv1::NUM_PARAMS; ++i) {
+		const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
+		// Filter out some non-randomizable parameters!...
+		if (index == drumkv1::GEN1_SAMPLE   ||
+		    index == drumkv1::GEN1_OFFSET   ||
+		    index == drumkv1::GEN1_OFFSET_1 ||
+		    index == drumkv1::GEN1_OFFSET_2 ||
+			index == drumkv1::GEN1_GROUP    ||
+			index == drumkv1::GEN1_COARSE   ||
+			index == drumkv1::GEN1_FINE     ||
+			index == drumkv1::GEN1_ENVTIME  ||
+			index == drumkv1::DCF1_ENABLED  ||
+			index == drumkv1::LFO1_ENABLED  ||
+			index == drumkv1::DCA1_ENABLED) 
+			continue;
+		if (index >= drumkv1::OUT1_WIDTH)
+			break;
+		drumkv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			std::normal_distribution<float> nd;
+			const float q = p * (pParam->maximum() - pParam->minimum());
+			float fValue = pParam->value();
+			if (drumkv1_param::paramFloat(index))
+				fValue += 0.5f * q * nd(re);
+			else
+				fValue = std::round(fValue + q * nd(re));
+			if (fValue < pParam->minimum())
+				fValue = pParam->minimum();
+			else
+			if (fValue > pParam->maximum())
+				fValue = pParam->maximum();
+			setParamValue(index, fValue);
+			updateParam(index, fValue);
+		}
+	}
+
+	m_ui.StatusBar->showMessage(tr("Randomize"), 5000);
+	updateDirtyPreset(true);
+}
+
+
 // Swap params A/B.
 void drumkv1widget::swapParams ( bool bOn )
 {
@@ -792,7 +891,7 @@ void drumkv1widget::swapParams ( bool bOn )
 		return;
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 #ifdef CONFIG_DEBUG
@@ -863,7 +962,7 @@ void drumkv1widget::updateParamValues ( uint32_t nparams )
 	resetSwapParams();
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	for (uint32_t i = 0; i < nparams; ++i) {
@@ -886,7 +985,7 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 	resetSwapParams();
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	for (uint32_t i = 0; i < nparams; ++i) {
@@ -1008,9 +1107,12 @@ void drumkv1widget::clearSample (void)
 // Sample file loader slot.
 void drumkv1widget::loadSample ( const QString& sFilename )
 {
-	loadSampleFile(QFileInfo(sFilename).canonicalFilePath());
+	const QFileInfo info(sFilename);
 
-	m_ui.StatusBar->showMessage(tr("Load sample: %1").arg(sFilename), 5000);
+	loadSampleFile(info.canonicalFilePath());
+
+	m_ui.StatusBar->showMessage(
+		tr("Load sample: %1").arg(info.fileName()), 5000);
 	updateDirtyPreset(true);
 }
 
@@ -1042,9 +1144,9 @@ void drumkv1widget::clearSampleFile (void)
 
 	drumkv1_ui *pDrumkUi = ui_instance();
 	if (pDrumkUi)
-		pDrumkUi->setSampleFile(NULL);
+		pDrumkUi->setSampleFile(nullptr);
 
-	updateSample(NULL);
+	updateSample(nullptr);
 }
 
 
@@ -1056,14 +1158,14 @@ void drumkv1widget::loadSampleFile ( const QString& sFilename )
 #endif
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	const int iCurrentNote = currentNote();
 	if (iCurrentNote < 0)
 		return;
 
-	if (pDrumkUi->element(iCurrentNote) == NULL) {
+	if (pDrumkUi->element(iCurrentNote) == nullptr) {
 		pDrumkUi->addElement(iCurrentNote);
 		pDrumkUi->setCurrentElement(iCurrentNote);
 	}
@@ -1083,11 +1185,11 @@ void drumkv1widget::updateSample ( drumkv1_sample *pSample, bool bDirty )
 
 	++m_iUpdate;
 	if (pSample) {
-		activateParamKnobs(pSample->filename() != NULL);
+		activateParamKnobs(pSample->filename() != nullptr);
 		updateOffset(pSample);
 	} else {
 		activateParamKnobs(false);
-		updateOffset(NULL);
+		updateOffset(nullptr);
 	}
 	--m_iUpdate;
 
@@ -1120,20 +1222,6 @@ QString drumkv1widget::noteName ( int note )
 
 	} s_notes[] = {
 
-		// Diatonic note map...
-		{  0, QT_TR_NOOP("C")     },
-		{  1, QT_TR_NOOP("C#/Db") },
-		{  2, QT_TR_NOOP("D")     },
-		{  3, QT_TR_NOOP("D#/Eb") },
-		{  4, QT_TR_NOOP("E")     },
-		{  5, QT_TR_NOOP("F")     },
-		{  6, QT_TR_NOOP("F#/Gb") },
-		{  7, QT_TR_NOOP("G")     },
-		{  8, QT_TR_NOOP("G#/Ab") },
-		{  9, QT_TR_NOOP("A")     },
-		{ 10, QT_TR_NOOP("A#/Bb") },
-		{ 11, QT_TR_NOOP("B")     },
-
 		// GM Drum note map...
 		{ 35, QT_TR_NOOP("Acoustic Bass Drum") },
 		{ 36, QT_TR_NOOP("Bass Drum 1") },
@@ -1183,7 +1271,7 @@ QString drumkv1widget::noteName ( int note )
 		{ 80, QT_TR_NOOP("Mute Triangle") },
 		{ 81, QT_TR_NOOP("Open Triangle") },
 
-		{  0, NULL }
+		{  0, nullptr }
 	};
 
 	static QHash<int, QString> s_names;
@@ -1192,7 +1280,7 @@ QString drumkv1widget::noteName ( int note )
 	if (s_names.isEmpty()) {
 		drumkv1_config *pConfig = drumkv1_config::getInstance();
 		if (pConfig && pConfig->bUseGMDrumNames) {
-			for (int i = 12; s_notes[i].name; ++i) {
+			for (int i = 0; s_notes[i].name; ++i) {
 				s_names.insert(s_notes[i].note,
 					QObject::tr(s_notes[i].name, "noteName"));
 			}
@@ -1204,7 +1292,7 @@ QString drumkv1widget::noteName ( int note )
 	if (iter != s_names.constEnd())
 		return iter.value();
 
-	return QString("%1 %2").arg(s_notes[note % 12].name).arg((note / 12) - 1);
+	return drumkv1_ui::noteName(note);
 }
 
 QString drumkv1widget::completeNoteName ( int note )
@@ -1226,7 +1314,7 @@ void drumkv1widget::refreshElements (void)
 	const bool bBlockSignals = m_ui.Elements->blockSignals(true);
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (m_ui.Elements->instance() == NULL)
+	if (m_ui.Elements->instance() == nullptr)
 		m_ui.Elements->setInstance(pDrumkUi);
 
 	int iCurrentNote = currentNote();
@@ -1261,6 +1349,7 @@ void drumkv1widget::clearElements (void)
 void drumkv1widget::activateElement ( bool bOpenSample )
 {
 	const int iCurrentNote = currentNote();
+	m_ui.StatusBar->keybd()->setNoteKey(iCurrentNote);
 	if (iCurrentNote < 0)
 		return;
 
@@ -1269,11 +1358,11 @@ void drumkv1widget::activateElement ( bool bOpenSample )
 #endif
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	drumkv1_element *element = pDrumkUi->element(iCurrentNote);
-	if (element == NULL && bOpenSample) {
+	if (element == nullptr && bOpenSample) {
 		element = pDrumkUi->addElement(iCurrentNote);
 		for (uint32_t i = 0; i < drumkv1::NUM_ELEMENT_PARAMS; ++i) {
 			const drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
@@ -1305,7 +1394,7 @@ void drumkv1widget::updateElement (void)
 	resetParamKnobs(drumkv1::NUM_ELEMENT_PARAMS);
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	const int iCurrentNote = pDrumkUi->currentElement();
@@ -1337,7 +1426,7 @@ void drumkv1widget::updateElement (void)
 		updateSample(pDrumkUi->sample());
 		refreshElements();
 	} else {
-		updateSample(NULL);
+		updateSample(nullptr);
 		resetParamValues(drumkv1::NUM_ELEMENT_PARAMS);
 	//	activateParamKnobs(false);
 	}
@@ -1426,7 +1515,7 @@ void drumkv1widget::offsetRangeChanged (void)
 // Update offset change status.
 void drumkv1widget::updateOffset ( drumkv1_sample *pSample, bool bDirty )
 {
-	if (pSample) {
+	if (pSample && pSample->filename()) {
 		const bool     bOffset    = pSample->isOffset();
 		const uint32_t iOffsetStart = pSample->offsetStart();
 		const uint32_t iOffsetEnd = pSample->offsetEnd();
@@ -1446,6 +1535,8 @@ void drumkv1widget::updateOffset ( drumkv1_sample *pSample, bool bDirty )
 		m_ui.Gen1Sample->setOffsetStart(iOffsetStart);
 		m_ui.Gen1Sample->setOffsetEnd(iOffsetEnd);
 		m_ui.Gen1Sample->setOffset(bOffset);
+		updateParam(drumkv1::GEN1_OFFSET_1, float(iOffsetStart) / float(nframes));
+		updateParam(drumkv1::GEN1_OFFSET_2, float(iOffsetEnd) / float(nframes));
 		if (bDirty) {
 			QString sMessage;
 			if (bOffset) {
@@ -1490,11 +1581,15 @@ void drumkv1widget::activateParamKnobs ( bool bEnabled )
 void drumkv1widget::activateParamKnobsGroupBox (
 	QGroupBox *pGroupBox, bool bEnabled )
 {
-	const QList<QWidget *>& children
-		= pGroupBox->findChildren<QWidget *> ();
-	QListIterator<QWidget *> iter(children);
-	while (iter.hasNext())
-		iter.next()->setEnabled(bEnabled);
+	if (pGroupBox->isCheckable()) {
+		pGroupBox->setEnabled(bEnabled);
+	} else {
+		const QList<QWidget *>& children
+			= pGroupBox->findChildren<QWidget *> ();
+		QListIterator<QWidget *> iter(children);
+		while (iter.hasNext())
+			iter.next()->setEnabled(bEnabled);
+	}
 }
 
 
@@ -1502,14 +1597,14 @@ void drumkv1widget::activateParamKnobsGroupBox (
 void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 {
 	QWidget *pSender = static_cast<QWidget *> (sender());
-	if (pSender == NULL)
+	if (pSender == nullptr)
 		return;
 
 	QMenu menu(this);
 	QAction *pAction;
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	drumkv1_element *element = NULL;
+	drumkv1_element *element = nullptr;
 	if (pDrumkUi) {
 		const int key = pDrumkUi->currentElement();
 		element = pDrumkUi->element(key);
@@ -1518,15 +1613,15 @@ void drumkv1widget::contextMenuRequest ( const QPoint& pos )
 	pAction = menu.addAction(
 		QIcon(":/images/fileOpen.png"),
 		tr("Open Sample..."), this, SLOT(openSample()));
-	pAction->setEnabled(pDrumkUi != NULL);
+	pAction->setEnabled(pDrumkUi != nullptr);
 	pAction = menu.addAction(
 		QIcon(":/images/playSample.png"),
 		tr("Play"), this, SLOT(playSample()));
-	pAction->setEnabled(element != NULL);
+	pAction->setEnabled(element != nullptr);
 	menu.addSeparator();
 	pAction = menu.addAction(
 		tr("Reset"), this, SLOT(resetElement()));
-	pAction->setEnabled(element != NULL);
+	pAction->setEnabled(element != nullptr);
 
 	QAbstractScrollArea *pAbstractScrollArea
 		= qobject_cast<QAbstractScrollArea *> (pSender);
@@ -1558,17 +1653,21 @@ void drumkv1widget::updateLoadPreset ( const QString& sPreset )
 void drumkv1widget::updateSchedNotify ( int stype, int sid )
 {
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("drumkv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (drumkv1_sched::Type(stype)) {
 	case drumkv1_sched::MidiIn:
-		if (sid >= 0)
-			m_ui.Elements->midiInLedNote(sid & 0x7f, (sid >> 7) & 0x7f);
+		if (sid >= 0) {
+			const int key = (sid & 0x7f);
+			const int vel = (sid >> 7) & 0x7f;
+			m_ui.Elements->midiInLedNote(key, vel);
+			m_ui.StatusBar->midiInNote(key, vel);
+		}
 		else
 		if (pDrumkUi->midiInCount() > 0) {
 			m_ui.StatusBar->midiInLed(true);
@@ -1612,6 +1711,19 @@ void drumkv1widget::updateSchedNotify ( int stype, int sid )
 }
 
 
+// Direct note-on/off slot.
+void drumkv1widget::directNoteOn ( int iNote, int iVelocity )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("drumkv1widget::directNoteOn(%d, %d)", iNote, iVelocity);
+#endif
+
+	drumkv1_ui *pDrumkUi = ui_instance();
+	if (pDrumkUi)
+		pDrumkUi->directNoteOn(iNote, iVelocity); // note-on!
+}
+
+
 // MIDI In LED timeout.
 void drumkv1widget::midiInLedTimeout (void)
 {
@@ -1623,7 +1735,7 @@ void drumkv1widget::midiInLedTimeout (void)
 void drumkv1widget::helpConfigure (void)
 {
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	drumkv1widget_config(pDrumkUi, this).exec();
@@ -1674,7 +1786,7 @@ void drumkv1widget::helpAbout (void)
 	sText += "</small>";
 	sText += "</p>\n";
 
-	QMessageBox::about(this, tr("About") + " " DRUMKV1_TITLE, sText);
+	QMessageBox::about(this, tr("About"), sText);
 }
 
 
@@ -1702,15 +1814,15 @@ void drumkv1widget::paramContextMenu ( const QPoint& pos )
 {
 	drumkv1widget_param *pParam
 		= qobject_cast<drumkv1widget_param *> (sender());
-	if (pParam == NULL)
+	if (pParam == nullptr)
 		return;
 
 	drumkv1_ui *pDrumkUi = ui_instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	drumkv1_controls *pControls = pDrumkUi->controls();
-	if (pControls == NULL)
+	if (pControls == nullptr)
 		return;
 
 	if (!pControls->enabled())
@@ -1735,7 +1847,7 @@ void drumkv1widget::spinboxContextMenu ( const QPoint& pos )
 {
 	drumkv1widget_spinbox *pSpinBox
 		= qobject_cast<drumkv1widget_spinbox *> (sender());
-	if (pSpinBox == NULL)
+	if (pSpinBox == nullptr)
 		return;
 
 	drumkv1widget_spinbox::Format format = pSpinBox->format();
@@ -1754,7 +1866,7 @@ void drumkv1widget::spinboxContextMenu ( const QPoint& pos )
 	pAction->setData(int(drumkv1widget_spinbox::Time));
 
 	pAction = menu.exec(pSpinBox->mapToGlobal(pos));
-	if (pAction == NULL)
+	if (pAction == nullptr)
 		return;
 
 	format = drumkv1widget_spinbox::Format(pAction->data().toInt());
