@@ -1,7 +1,7 @@
 // drumkv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -89,6 +89,9 @@ public slots:
 	void loadPreset(const QString& sFilename);
 	void savePreset(const QString& sFilename);
 
+	// Direct note-on/off slot.
+	void directNoteOn(int iNote, int iVelocity);
+
 protected slots:
 
 	// Preset renewal.
@@ -130,6 +133,9 @@ protected slots:
 	// Reset param knobs to default value.
 	void resetParams();
 
+	// Randomize params (partial).
+	void randomParams();
+
 	// Swap params A/B.
 	void swapParams(bool bOn);
 
@@ -242,7 +248,7 @@ class drumkv1widget_sched : public QObject
 public:
 
 	// ctor.
-	drumkv1widget_sched(drumkv1 *pDrumk, QObject *pParent = NULL)
+	drumkv1widget_sched(drumkv1 *pDrumk, QObject *pParent = nullptr)
 		: QObject(pParent), m_notifier(pDrumk, this) {}
 
 signals:
@@ -253,12 +259,12 @@ signals:
 protected:
 
 	// Notififier visitor.
-	class Notifier : public drumkv1_sched_notifier
+	class Notifier : public drumkv1_sched::Notifier
 	{
 	public:
 
 		Notifier(drumkv1 *pDrumk, drumkv1widget_sched *pSched)
-			: drumkv1_sched_notifier(pDrumk), m_pSched(pSched) {}
+			: drumkv1_sched::Notifier(pDrumk), m_pSched(pSched) {}
 
 		void notify(drumkv1_sched::Type stype, int sid) const
 			{ m_pSched->emit_notify(stype, sid); }
