@@ -1,7 +1,7 @@
 // drumkv1widget_config.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,6 +22,8 @@
 #include "drumkv1widget_config.h"
 #include "drumkv1widget_param.h"
 
+#include "drumkv1widget_palette.h"
+
 #include "drumkv1_ui.h"
 
 #include "drumkv1_controls.h"
@@ -42,6 +44,10 @@
 #include <QStyleFactory>
 
 
+// Default (empty/blank) name.
+static const char *g_pszDefName = QT_TRANSLATE_NOOP("drumkv1widget_config", "(default)");
+
+
 //----------------------------------------------------------------------------
 // drumkv1widget_config -- UI wrapper form.
 
@@ -54,17 +60,22 @@ drumkv1widget_config::drumkv1widget_config (
 	// Setup UI struct...
 	m_ui.setupUi(this);
 
-	// Custom style themes...
-	//m_ui.CustomStyleThemeComboBox->clear();
-	//m_ui.CustomStyleThemeComboBox->addItem(tr("(default)"));
-	m_ui.CustomStyleThemeComboBox->addItems(QStyleFactory::keys());
-
 	// Note names.
 	QStringList notes;
 	for (int note = 0; note < 128; ++note)
 		notes << drumkv1_ui::noteName(note);
 	m_ui.TuningRefNoteComboBox->insertItems(0, notes);
 
+	// Tuning specifics setup...
+	m_ui.TuningTabBar->addTab(tr("&Global"));
+	m_ui.TuningTabBar->addTab(tr("&Instance"));
+
+	// Dialog dirty flags.
+	m_iDirtyTuning   = 0;
+	m_iDirtyControls = 0;
+	m_iDirtyPrograms = 0;
+	m_iDirtyOptions  = 0;
+
 	// Setup options...
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
 	if (pConfig && m_pDrumkUi) {
@@ -73,14 +84,12 @@ drumkv1widget_config::drumkv1widget_config (
 		m_ui.UseNativeDialogsCheckBox->setChecked(pConfig->bUseNativeDialogs);
 		m_ui.KnobDialModeComboBox->setCurrentIndex(pConfig->iKnobDialMode);
 		m_ui.KnobEditModeComboBox->setCurrentIndex(pConfig->iKnobEditMode);
-		int iCustomStyleTheme = 0;
-		if (!pConfig->sCustomStyleTheme.isEmpty())
-			iCustomStyleTheme = m_ui.CustomStyleThemeComboBox->findText(
-				pConfig->sCustomStyleTheme);
-		m_ui.CustomStyleThemeComboBox->setCurrentIndex(iCustomStyleTheme);
-		m_ui.CustomStyleThemeTextLabel->setEnabled(!bPlugin);
-		m_ui.CustomStyleThemeComboBox->setEnabled(!bPlugin);
+		m_ui.FrameTimeFormatComboBox->setCurrentIndex(pConfig->iFrameTimeFormat);
+		m_ui.RandomizePercentSpinBox->setValue(pConfig->fRandomizePercent);
 		m_ui.UseGMDrumNamesCheckBox->setChecked(pConfig->bUseGMDrumNames);
+		// Custom display options (only for no-plugin forms)...
+		resetCustomColorThemes(pConfig->sCustomColorTheme);
+		resetCustomStyleThemes(pConfig->sCustomStyleTheme);
 		// Load controllers database...
 		drumkv1_controls *pControls = m_pDrumkUi->controls();
 		if (pControls) {
@@ -100,15 +109,9 @@ drumkv1widget_config::drumkv1widget_config (
 		loadComboBoxHistory(m_ui.TuningScaleFileComboBox);
 		loadComboBoxHistory(m_ui.TuningKeyMapFileComboBox);
 		// Micro-tonal tuning settings...
-		m_ui.TuningEnabledCheckBox->setChecked(pConfig->bTuningEnabled);
-		m_ui.TuningRefNoteComboBox->setCurrentIndex(pConfig->iTuningRefNote);
-		m_ui.TuningRefPitchSpinBox->setValue(double(pConfig->fTuningRefPitch));
-		setComboBoxCurrentItem(
-			m_ui.TuningScaleFileComboBox,
-			QFileInfo(pConfig->sTuningScaleFile));
-		setComboBoxCurrentItem(
-			m_ui.TuningKeyMapFileComboBox,
-			QFileInfo(pConfig->sTuningKeyMapFile));
+		const int iTuningTab = (m_pDrumkUi->isTuningEnabled() ? 1 : 0);
+		m_ui.TuningTabBar->setCurrentIndex(iTuningTab);
+		tuningTabChanged(iTuningTab);
 	}
 
 	// Signal/slots connections...
@@ -172,6 +175,9 @@ drumkv1widget_config::drumkv1widget_config (
 		SLOT(programsContextMenuRequested(const QPoint&)));
 
 	// Tuning slots...
+	QObject::connect(m_ui.TuningTabBar,
+		SIGNAL(currentChanged(int)),
+		SLOT(tuningTabChanged(int)));
 	QObject::connect(m_ui.TuningEnabledCheckBox,
 		SIGNAL(toggled(bool)),
 		SLOT(tuningChanged()));
@@ -210,12 +216,24 @@ drumkv1widget_config::drumkv1widget_config (
 	QObject::connect(m_ui.KnobEditModeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.CustomColorThemeComboBox,
+		SIGNAL(activated(int)),
+		SLOT(optionsChanged()));
+	QObject::connect(m_ui.CustomColorThemeToolButton,
+		SIGNAL(clicked()),
+		SLOT(editCustomColorThemes()));
 	QObject::connect(m_ui.CustomStyleThemeComboBox,
 		SIGNAL(activated(int)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.FrameTimeFormatComboBox,
+		SIGNAL(activated(int)),
+		SLOT(optionsChanged()));
 	QObject::connect(m_ui.UseGMDrumNamesCheckBox,
 		SIGNAL(toggled(bool)),
 		SLOT(optionsChanged()));
+	QObject::connect(m_ui.RandomizePercentSpinBox,
+		SIGNAL(valueChanged(double)),
+		SLOT(optionsChanged()));
 
 	// Dialog commands...
 	QObject::connect(m_ui.DialogButtonBox,
@@ -225,12 +243,6 @@ drumkv1widget_config::drumkv1widget_config (
 		SIGNAL(rejected()),
 		SLOT(reject()));
 
-	// Dialog dirty flags.
-	m_iDirtyTuning   = 0;
-	m_iDirtyControls = 0;
-	m_iDirtyPrograms = 0;
-	m_iDirtyOptions  = 0;
-
 	// Done.
 	stabilize();
 }
@@ -292,7 +304,7 @@ void drumkv1widget_config::controlsContextMenuRequested ( const QPoint& pos )
 	QMenu menu(this);
 	QAction *pAction;
 
-	bool bEnabled = (m_pDrumkUi && m_pDrumkUi->controls() != NULL);
+	bool bEnabled = (m_pDrumkUi && m_pDrumkUi->controls() != nullptr);
 
 	pAction = menu.addAction(QIcon(":/images/drumkv1_preset.png"),
 		tr("&Add Controller"), this, SLOT(controlsAddItem()));
@@ -300,7 +312,7 @@ void drumkv1widget_config::controlsContextMenuRequested ( const QPoint& pos )
 
 	menu.addSeparator();
 
-	bEnabled = bEnabled && (pItem != NULL);
+	bEnabled = bEnabled && (pItem != nullptr);
 
 	pAction = menu.addAction(QIcon(":/images/presetEdit.png"),
 		tr("&Edit"), this, SLOT(controlsEditItem()));
@@ -387,7 +399,7 @@ void drumkv1widget_config::programsContextMenuRequested ( const QPoint& pos )
 	QMenu menu(this);
 	QAction *pAction;
 
-	bool bEnabled = (m_pDrumkUi && m_pDrumkUi->programs() != NULL);
+	bool bEnabled = (m_pDrumkUi && m_pDrumkUi->programs() != nullptr);
 
 	pAction = menu.addAction(QIcon(":/images/presetBank.png"),
 		tr("Add &Bank"), this, SLOT(programsAddBankItem()));
@@ -399,7 +411,7 @@ void drumkv1widget_config::programsContextMenuRequested ( const QPoint& pos )
 
 	menu.addSeparator();
 
-	bEnabled = bEnabled && (pItem != NULL);
+	bEnabled = bEnabled && (pItem != nullptr);
 
 	pAction = menu.addAction(QIcon(":/images/presetEdit.png"),
 		tr("&Edit"), this, SLOT(programsEditItem()));
@@ -448,6 +460,57 @@ void drumkv1widget_config::programsActivated (void)
 
 
 // tuning command slots
+void drumkv1widget_config::tuningTabChanged ( int iTuningTab )
+{
+	// Prevent loss of some tuning changes here...
+	if (m_iDirtyTuning > 0 &&
+		QMessageBox::warning(this,
+			tr("Warning"),
+			tr("%1 tuning settings have been changed.\n\n"
+			"Do you want to discard the changes?")
+			.arg(m_ui.TuningTabBar->tabText(1 - iTuningTab).remove('&')),
+			QMessageBox::Discard | QMessageBox::Cancel)
+			== QMessageBox::Cancel) {
+		const bool bBlockSignals = m_ui.TuningTabBar->blockSignals(true);
+		m_ui.TuningTabBar->setCurrentIndex(1 - iTuningTab);
+		m_ui.TuningTabBar->blockSignals(bBlockSignals);
+		return;
+	}
+
+	if (iTuningTab == 0) {
+		// Global (default) scope...
+		drumkv1_config *pConfig = drumkv1_config::getInstance();
+		if (pConfig) {
+			m_ui.TuningEnabledCheckBox->setChecked(pConfig->bTuningEnabled);
+			m_ui.TuningRefNoteComboBox->setCurrentIndex(pConfig->iTuningRefNote);
+			m_ui.TuningRefPitchSpinBox->setValue(double(pConfig->fTuningRefPitch));
+			setComboBoxCurrentItem(
+				m_ui.TuningScaleFileComboBox,
+				QFileInfo(pConfig->sTuningScaleFile));
+			setComboBoxCurrentItem(
+				m_ui.TuningKeyMapFileComboBox,
+				QFileInfo(pConfig->sTuningKeyMapFile));
+		}
+	}
+	else
+	if (m_pDrumkUi) {
+		// Instance scope...
+		m_ui.TuningEnabledCheckBox->setChecked(m_pDrumkUi->isTuningEnabled());
+		m_ui.TuningRefNoteComboBox->setCurrentIndex(m_pDrumkUi->tuningRefNote());
+		m_ui.TuningRefPitchSpinBox->setValue(double(m_pDrumkUi->tuningRefPitch()));
+		setComboBoxCurrentItem(
+			m_ui.TuningScaleFileComboBox,
+			QFileInfo(QString::fromUtf8(m_pDrumkUi->tuningScaleFile())));
+		setComboBoxCurrentItem(
+			m_ui.TuningKeyMapFileComboBox,
+			QFileInfo(QString::fromUtf8(m_pDrumkUi->tuningKeyMapFile())));
+	}
+
+	// Reset tuning dirty flag...
+	m_iDirtyTuning = 0;
+}
+
+
 void drumkv1widget_config::tuningRefNoteClicked (void)
 {
 	m_ui.TuningRefNoteComboBox->setCurrentIndex(69);
@@ -460,28 +523,28 @@ void drumkv1widget_config::tuningRefNoteClicked (void)
 void drumkv1widget_config::tuningScaleFileClicked (void)
 {
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	QString sTuningScaleFile = m_ui.TuningScaleFileComboBox->currentText();
 
 	const QString  sExt("scl");
-	const QString& sTitle  = tr("Open Scale File") + " - " DRUMKV1_TITLE;
+	const QString& sTitle  = tr("Open Scale File");
 
 	QStringList filters;
 	filters.append(tr("Scale files (*.%1)").arg(sExt));
 	filters.append(tr("All files (*.*)"));
 	const QString& sFilter = filters.join(";;");
 
-	QWidget *pParentWidget = NULL;
+	QWidget *pParentWidget = nullptr;
 	QFileDialog::Options options = 0;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
 		pParentWidget = QWidget::window();
 	}
-#if 1//QT_VERSION < 0x040400
+#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 	sTuningScaleFile = QFileDialog::getOpenFileName(pParentWidget,
-		sTitle, pConfig->sTuningScaleDir, sFilter, NULL, options);
+		sTitle, pConfig->sTuningScaleDir, sFilter, nullptr, options);
 #else
 	QFileDialog fileDialog(pParentWidget,
 		sTitle, sTuningScaleFile, sFilter);
@@ -509,28 +572,28 @@ void drumkv1widget_config::tuningScaleFileClicked (void)
 void drumkv1widget_config::tuningKeyMapFileClicked (void)
 {
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	QString sTuningKeyMapFile = m_ui.TuningKeyMapFileComboBox->currentText();
 
 	const QString  sExt("kbm");
-	const QString& sTitle  = tr("Open Key Map File") + " - " DRUMKV1_TITLE;
+	const QString& sTitle  = tr("Open Key Map File");
 
 	QStringList filters;
 	filters.append(tr("Key Map files (*.%1)").arg(sExt));
 	filters.append(tr("All files (*.*)"));
 	const QString& sFilter = filters.join(";;");
 
-	QWidget *pParentWidget = NULL;
+	QWidget *pParentWidget = nullptr;
 	QFileDialog::Options options = 0;
 	if (pConfig->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
 		pParentWidget = QWidget::window();
 	}
-#if 1//QT_VERSION < 0x040400
+#if 1//QT_VERSION < QT_VERSION_CHECK(4, 4, 0)
 	sTuningKeyMapFile = QFileDialog::getOpenFileName(pParentWidget,
-		sTitle, pConfig->sTuningKeyMapDir, sFilter, NULL, options);
+		sTitle, pConfig->sTuningKeyMapDir, sFilter, nullptr, options);
 #else
 	QFileDialog fileDialog(pParentWidget,
 		sTitle, sTuningScaleFile, sFilter);
@@ -576,19 +639,19 @@ void drumkv1widget_config::optionsChanged (void)
 void drumkv1widget_config::stabilize (void)
 {
 	QTreeWidgetItem *pItem = m_ui.ControlsTreeWidget->currentItem();
-	bool bEnabled = (m_pDrumkUi && m_pDrumkUi->controls() != NULL);
+	bool bEnabled = (m_pDrumkUi && m_pDrumkUi->controls() != nullptr);
 	m_ui.ControlsAddItemToolButton->setEnabled(bEnabled);
-	bEnabled = bEnabled && (pItem != NULL);
+	bEnabled = bEnabled && (pItem != nullptr);
 	m_ui.ControlsEditToolButton->setEnabled(bEnabled);
 	m_ui.ControlsDeleteToolButton->setEnabled(bEnabled);
 
 	pItem = m_ui.ProgramsTreeWidget->currentItem();
-	bEnabled = (m_pDrumkUi && m_pDrumkUi->programs() != NULL);
+	bEnabled = (m_pDrumkUi && m_pDrumkUi->programs() != nullptr);
 	m_ui.ProgramsPreviewCheckBox->setEnabled(
 		bEnabled && m_ui.ProgramsEnabledCheckBox->isChecked());
 	m_ui.ProgramsAddBankToolButton->setEnabled(bEnabled);
 	m_ui.ProgramsAddItemToolButton->setEnabled(bEnabled);
-	bEnabled = bEnabled && (pItem != NULL);
+	bEnabled = bEnabled && (pItem != nullptr);
 	m_ui.ProgramsEditToolButton->setEnabled(bEnabled);
 	m_ui.ProgramsDeleteToolButton->setEnabled(bEnabled);
 
@@ -622,13 +685,27 @@ void drumkv1widget_config::accept (void)
 
 	if (m_iDirtyTuning > 0 && pConfig && m_pDrumkUi) {
 		// Micro-tonal tuning settings...
-		pConfig->bTuningEnabled = m_ui.TuningEnabledCheckBox->isChecked();
-		pConfig->iTuningRefNote = m_ui.TuningRefNoteComboBox->currentIndex();
-		pConfig->fTuningRefPitch = float(m_ui.TuningRefPitchSpinBox->value());
-		pConfig->sTuningScaleFile = comboBoxCurrentItem(m_ui.TuningScaleFileComboBox);
-		pConfig->sTuningKeyMapFile = comboBoxCurrentItem(m_ui.TuningKeyMapFileComboBox);
+		if (m_ui.TuningTabBar->currentIndex() == 0) {
+			// Global (default) scope...
+			pConfig->bTuningEnabled = m_ui.TuningEnabledCheckBox->isChecked();
+			pConfig->iTuningRefNote = m_ui.TuningRefNoteComboBox->currentIndex();
+			pConfig->fTuningRefPitch = float(m_ui.TuningRefPitchSpinBox->value());
+			pConfig->sTuningScaleFile = comboBoxCurrentItem(m_ui.TuningScaleFileComboBox);
+			pConfig->sTuningKeyMapFile = comboBoxCurrentItem(m_ui.TuningKeyMapFileComboBox);
+		} else {
+			m_pDrumkUi->setTuningEnabled(
+				m_ui.TuningEnabledCheckBox->isChecked());
+			m_pDrumkUi->setTuningRefNote(
+				m_ui.TuningRefNoteComboBox->currentIndex());
+			m_pDrumkUi->setTuningRefPitch(
+				float(m_ui.TuningRefPitchSpinBox->value()));
+			m_pDrumkUi->setTuningScaleFile(comboBoxCurrentItem(
+				m_ui.TuningScaleFileComboBox).toUtf8().constData());
+			m_pDrumkUi->setTuningKeyMapFile(comboBoxCurrentItem(
+				m_ui.TuningKeyMapFileComboBox).toUtf8().constData());
+		}
 		// Reset/update micro-tonal tuning...
-		m_pDrumkUi->updateTuning();
+		m_pDrumkUi->resetTuning();
 		// Save other conveniency options...
 		saveComboBoxHistory(m_ui.TuningScaleFileComboBox);
 		saveComboBoxHistory(m_ui.TuningKeyMapFileComboBox);
@@ -669,16 +746,32 @@ void drumkv1widget_config::accept (void)
 		pConfig->iKnobEditMode = m_ui.KnobEditModeComboBox->currentIndex();
 		drumkv1widget_edit::setEditMode(
 			drumkv1widget_edit::EditMode(pConfig->iKnobEditMode));
+		const QString sOldCustomColorTheme = pConfig->sCustomColorTheme;
+		if (m_ui.CustomColorThemeComboBox->currentIndex() > 0)
+			pConfig->sCustomColorTheme = m_ui.CustomColorThemeComboBox->currentText();
+		else
+			pConfig->sCustomColorTheme.clear();
 		const QString sOldCustomStyleTheme = pConfig->sCustomStyleTheme;
 		if (m_ui.CustomStyleThemeComboBox->currentIndex() > 0)
 			pConfig->sCustomStyleTheme = m_ui.CustomStyleThemeComboBox->currentText();
 		else
 			pConfig->sCustomStyleTheme.clear();
+		const int iOldFrameTimeFormat = pConfig->iFrameTimeFormat;
 		const bool bOldUseGMDrumNames = pConfig->bUseGMDrumNames;
+		pConfig->iFrameTimeFormat = m_ui.FrameTimeFormatComboBox->currentIndex();
+		pConfig->fRandomizePercent = float(m_ui.RandomizePercentSpinBox->value());
 		pConfig->bUseGMDrumNames = m_ui.UseGMDrumNamesCheckBox->isChecked();
 		int iNeedRestart = 0;
-		if (!pConfig->bUseGMDrumNames && bOldUseGMDrumNames)
-			++iNeedRestart;
+ 		if (pConfig->sCustomColorTheme != sOldCustomColorTheme) {
+			if (pConfig->sCustomColorTheme.isEmpty()) {
+				++iNeedRestart;
+			} else {
+				QPalette pal;
+				if (drumkv1widget_palette::namedPalette(
+						pConfig, pConfig->sCustomColorTheme, pal))
+					QApplication::setPalette(pal);
+			}
+		}
  		if (pConfig->sCustomStyleTheme != sOldCustomStyleTheme) {
 			if (pConfig->sCustomStyleTheme.isEmpty()) {
 				++iNeedRestart;
@@ -687,10 +780,14 @@ void drumkv1widget_config::accept (void)
 					QStyleFactory::create(pConfig->sCustomStyleTheme));
 			}
  		}
+		if (pConfig->iFrameTimeFormat != iOldFrameTimeFormat)
+			++iNeedRestart;
+		if (!pConfig->bUseGMDrumNames && bOldUseGMDrumNames)
+			++iNeedRestart;
 		// Show restart message if needed...
  		if (iNeedRestart > 0) {
 			QMessageBox::information(this,
-				tr("Information") + " - " DRUMKV1_TITLE,
+				tr("Information"),
 				tr("Some settings may be only effective\n"
 				"next time you start this application."));
 		}
@@ -717,7 +814,7 @@ void drumkv1widget_config::reject (void)
 		if (m_ui.DialogButtonBox->button(QDialogButtonBox::Ok)->isEnabled())
 			buttons |= QMessageBox::Apply;
 		switch (QMessageBox::warning(this,
-			tr("Warning") + " - " DRUMKV1_TITLE,
+			tr("Warning"),
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			buttons)) {
@@ -736,11 +833,86 @@ void drumkv1widget_config::reject (void)
 }
 
 
+// Custom color palette theme manager.
+void drumkv1widget_config::editCustomColorThemes (void)
+{
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig == nullptr)
+		return;
+
+	drumkv1widget_palette form(this);
+	form.setSettings(pConfig);
+
+	QString sCustomColorTheme;
+	int iDirtyCustomColorTheme = 0;
+
+	const int iCustomColorTheme
+		= m_ui.CustomColorThemeComboBox->currentIndex();
+	if (iCustomColorTheme > 0) {
+		sCustomColorTheme = m_ui.CustomColorThemeComboBox->itemText(iCustomColorTheme);
+		form.setPaletteName(sCustomColorTheme);
+	}
+
+	if (form.exec() == QDialog::Accepted) {
+		sCustomColorTheme = form.paletteName();
+		++iDirtyCustomColorTheme;
+	}
+
+	if (iDirtyCustomColorTheme > 0 || form.isDirty()) {
+		resetCustomColorThemes(sCustomColorTheme);
+		optionsChanged();
+	}
+}
+
+
+// Custom color palette themes settler.
+void drumkv1widget_config::resetCustomColorThemes (
+	const QString& sCustomColorTheme )
+{
+	m_ui.CustomColorThemeComboBox->clear();
+	m_ui.CustomColorThemeComboBox->addItem(
+		tr(g_pszDefName));
+	drumkv1_config *pConfig = drumkv1_config::getInstance();
+	if (pConfig) m_ui.CustomColorThemeComboBox->addItems(
+		drumkv1widget_palette::namedPaletteList(pConfig));
+
+	int iCustomColorTheme = 0;
+	if (!sCustomColorTheme.isEmpty()) {
+		iCustomColorTheme = m_ui.CustomColorThemeComboBox->findText(
+			sCustomColorTheme);
+		if (iCustomColorTheme < 0)
+			iCustomColorTheme = 0;
+	}
+	m_ui.CustomColorThemeComboBox->setCurrentIndex(iCustomColorTheme);
+}
+
+
+// Custom widget style themes settler.
+void drumkv1widget_config::resetCustomStyleThemes (
+	const QString& sCustomStyleTheme )
+{
+	m_ui.CustomStyleThemeComboBox->clear();
+	m_ui.CustomStyleThemeComboBox->addItem(
+		tr(g_pszDefName));
+	m_ui.CustomStyleThemeComboBox->addItems(
+		QStyleFactory::keys());
+
+	int iCustomStyleTheme = 0;
+	if (!sCustomStyleTheme.isEmpty()) {
+		iCustomStyleTheme = m_ui.CustomStyleThemeComboBox->findText(
+			sCustomStyleTheme);
+		if (iCustomStyleTheme < 0)
+			iCustomStyleTheme = 0;
+	}
+	m_ui.CustomStyleThemeComboBox->setCurrentIndex(iCustomStyleTheme);
+}
+
+
 // Combo box history persistence helper implementation.
 void drumkv1widget_config::loadComboBoxHistory ( QComboBox *pComboBox )
 {
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	// Load combobox list from configuration settings file...
@@ -764,7 +936,7 @@ void drumkv1widget_config::loadComboBoxHistory ( QComboBox *pComboBox )
 void drumkv1widget_config::saveComboBoxHistory ( QComboBox *pComboBox )
 {
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
-	if (pConfig == NULL)
+	if (pConfig == nullptr)
 		return;
 
 	// Save combobox list to configuration settings file...
