@@ -1,7 +1,7 @@
 // drumkv1widget_config.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -58,6 +58,8 @@ public:
 protected slots:
 
 	// command slots.
+	void editCustomColorThemes();
+
 	void controlsAddItem();
 	void controlsEditItem();
 	void controlsDeleteItem();
@@ -78,6 +80,7 @@ protected slots:
 	void controlsEnabled(bool);
 	void programsEnabled(bool);
 
+	void tuningTabChanged(int);
 	void tuningRefNoteClicked();
 	void tuningScaleFileClicked();
 	void tuningKeyMapFileClicked();
@@ -93,6 +96,10 @@ protected slots:
 
 protected:
 
+	// Custom color/style themes settlers.
+	void resetCustomColorThemes(const QString& sCustomColorTheme);
+	void resetCustomStyleThemes(const QString& sCustomColorTheme);
+
 	// Combo box history persistence helper prototypes.
 	void loadComboBoxHistory(QComboBox *pComboBox);
 	void saveComboBoxHistory(QComboBox *pComboBox);
