@@ -1,7 +1,7 @@
 // drumkv1widget_elements.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -136,7 +136,7 @@ QVariant drumkv1widget_elements_model::data (
 QModelIndex drumkv1widget_elements_model::index (
 	int row, int column, const QModelIndex& /*parent*/) const
 {
-	return createIndex(row, column, (m_pDrumkUi ? m_pDrumkUi->element(row) : NULL));
+	return createIndex(row, column, (m_pDrumkUi ? m_pDrumkUi->element(row) : nullptr));
 }
 
 
@@ -167,7 +167,6 @@ void drumkv1widget_elements_model::midiInLedNote ( int key, int vel )
 	}
 	else
 	if (m_notes_on[key] > 0) {
-		m_notes_off.append(key);
 		QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
 	}
 }
@@ -175,21 +174,19 @@ void drumkv1widget_elements_model::midiInLedNote ( int key, int vel )
 
 void drumkv1widget_elements_model::midiInLedTimeout (void)
 {
-	QListIterator<int> iter(m_notes_off);
-	while (iter.hasNext()) {
-		const int key = iter.next();
-		midiInLedUpdate(key);
-		m_notes_on[key] = 0;
+	for (int key = 0; key < MAX_NOTES; ++key) {
+		if (m_notes_on[key] > 0) {
+			m_notes_on[key] = 0;
+			midiInLedUpdate(key);
+		}
 	}
-
-	m_notes_off.clear();
 }
 
 
 void drumkv1widget_elements_model::midiInLedUpdate ( int key )
 {
 	const QModelIndex& index = drumkv1widget_elements_model::index(key, 0);
-#if QT_VERSION >= 0x050100
+#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
 	emit dataChanged(index, index, QVector<int>() << Qt::DecorationRole);
 #else
 	emit dataChanged(index, index);
@@ -199,7 +196,7 @@ void drumkv1widget_elements_model::midiInLedUpdate ( int key )
 
 void drumkv1widget_elements_model::reset (void)
 {
-#if QT_VERSION < 0x050000
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	QAbstractItemModel::reset();
 #else
 	QAbstractItemModel::beginResetModel();
@@ -255,8 +252,8 @@ int drumkv1widget_elements_model::columnAlignment( int /*column*/ ) const
 
 // Constructor.
 drumkv1widget_elements::drumkv1widget_elements ( QWidget *pParent )
-	: QTreeView(pParent), m_pModel(NULL),
-		m_pDragSample(NULL), m_iDirectNoteOn(-1)
+	: QTreeView(pParent), m_pModel(nullptr),
+		m_pDragSample(nullptr), m_iDirectNoteOn(-1), m_iDirectNoteOnVelocity(64)
 {
 	resetDragState();
 }
@@ -309,7 +306,7 @@ void drumkv1widget_elements::setInstance ( drumkv1_ui *pDrumkUi )
 
 drumkv1_ui *drumkv1widget_elements::instance (void) const
 {
-	return (m_pModel ? m_pModel->instance() : NULL);
+	return (m_pModel ? m_pModel->instance() : nullptr);
 }
 
 
@@ -389,7 +386,7 @@ void drumkv1widget_elements::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 
 	directNoteOff();
 
-	m_pDragSample = NULL;
+	m_pDragSample = nullptr;
 	resetDragState();
 }
 
@@ -451,7 +448,7 @@ void drumkv1widget_elements::resetDragState (void)
 // Refreshner.
 void drumkv1widget_elements::refresh (void)
 {
-	if (m_pModel == NULL)
+	if (m_pModel == nullptr)
 		return;
 
 	QItemSelectionModel *pSelectionModel = QTreeView::selectionModel();
@@ -483,35 +480,33 @@ void drumkv1widget_elements::midiInLedNote ( int key, int vel )
 // Direct note-on/off methods.
 void drumkv1widget_elements::directNoteOn ( int key )
 {
-	if (m_pModel == NULL || key < 0)
+	if (m_pModel == nullptr || key < 0)
 		return;
 
 	drumkv1_ui *pDrumkUi = m_pModel->instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
-	drumkv1_sample *pSample = pDrumkUi->sample();
-	if (pSample == NULL)
-		return;
-
-	const float v = pDrumkUi->paramValue(drumkv1::DEF1_VELOCITY);
-	const int vel = int(79.375f * v + 47.625f) & 0x7f;
-	pDrumkUi->directNoteOn(key, vel); // note-on!
 	m_iDirectNoteOn = key;
 
-	const float srate_ms = 0.001f * pSample->sampleRate();
-	const int timeout_ms = int(float(pSample->length() >> 1) / srate_ms);
-	QTimer::singleShot(timeout_ms, this, SLOT(directNoteOff()));
+	pDrumkUi->directNoteOn(m_iDirectNoteOn, m_iDirectNoteOnVelocity);
+
+	drumkv1_sample *pSample = pDrumkUi->sample();
+	if (pSample) {
+		const float srate_ms = 0.001f * pSample->sampleRate();
+		const int timeout_ms = int(float(pSample->length() >> 1) / srate_ms);
+		QTimer::singleShot(timeout_ms, this, SLOT(directNoteOff()));
+	}
 }
 
 
 void drumkv1widget_elements::directNoteOff (void)
 {
-	if (m_pModel == NULL || m_iDirectNoteOn < 0)
+	if (m_pModel == nullptr || m_iDirectNoteOn < 0)
 		return;
 
 	drumkv1_ui *pDrumkUi = m_pModel->instance();
-	if (pDrumkUi == NULL)
+	if (pDrumkUi == nullptr)
 		return;
 
 	pDrumkUi->directNoteOn(m_iDirectNoteOn, 0); // note-off!
@@ -520,4 +515,17 @@ void drumkv1widget_elements::directNoteOff (void)
 }
 
 
+// Direct note-on velocity accessors.
+void drumkv1widget_elements::setDirectNoteOnVelocity ( int vel )
+{
+	m_iDirectNoteOnVelocity = vel;
+}
+
+
+int drumkv1widget_elements::directNoteOnVelocity (void) const
+{
+	return m_iDirectNoteOnVelocity;
+}
+
+
 // end of drumkv1widget_elements.cpp
