@@ -1,7 +1,7 @@
 // drumkv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 
 #include "drumkv1widget_jack.h"
 
+#include "drumkv1widget_palette.h"
+
 #include "drumkv1_jack.h"
 
 #ifdef CONFIG_NSM
@@ -43,7 +45,7 @@
 #endif
 #endif
 
-#if QT_VERSION < 0x050000
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 #define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt4/plugins"
 #else
 #define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt5/plugins"
@@ -58,30 +60,43 @@
 drumkv1widget_jack::drumkv1widget_jack ( drumkv1_jack *pDrumk )
 	: drumkv1widget(), m_pDrumk(pDrumk)
 	#ifdef CONFIG_NSM
-		, m_pNsmClient(NULL)
+		, m_pNsmClient(nullptr)
 	#endif
 {
 	// Special style paths...
 	if (QDir(CONFIG_PLUGINSDIR).exists())
 		QApplication::addLibraryPath(CONFIG_PLUGINSDIR);
 
-	// Custom style theme...
+	// Custom color/style themes...
 	drumkv1_config *pConfig = drumkv1_config::getInstance();
-	if (pConfig && !pConfig->sCustomStyleTheme.isEmpty())
-		QApplication::setStyle(QStyleFactory::create(pConfig->sCustomStyleTheme));
+	if (pConfig) {
+		if (!pConfig->sCustomColorTheme.isEmpty()) {
+			QPalette pal;
+			if (drumkv1widget_palette::namedPalette(
+					pConfig, pConfig->sCustomColorTheme, pal))
+				QApplication::setPalette(pal);
+		}
+		if (!pConfig->sCustomStyleTheme.isEmpty()) {
+			QApplication::setStyle(
+				QStyleFactory::create(pConfig->sCustomStyleTheme));
+		}
+	}
 
 	// Initialize (user) interface stuff...
 	m_pDrumkUi = new drumkv1_ui(m_pDrumk, false);
 
-	// May initialize the scheduler/work notifier.
-	openSchedNotifier();
+	// Initialise preset stuff...
+	clearPreset();
 
-	// Initialize preset stuff...
-	//initPreset();
+	// Initial update, always...
 	refreshElements();
 	activateElement();
 
-	updateParamValues(drumkv1::NUM_PARAMS);
+	resetParamValues(drumkv1::NUM_PARAMS);
+	resetParamKnobs(drumkv1::NUM_PARAMS);
+
+	// May initialize the scheduler/work notifier.
+	openSchedNotifier();
 }
 
 
@@ -157,7 +172,7 @@ void drumkv1widget_jack::closeEvent ( QCloseEvent *pCloseEvent )
 // Optional GUI handlers.
 void drumkv1widget_jack::showEvent ( QShowEvent *pShowEvent )
 {
-	QWidget::showEvent(pShowEvent);
+	drumkv1widget::showEvent(pShowEvent);
 
 	if (m_pNsmClient)
 		m_pNsmClient->visible(true);
@@ -168,7 +183,7 @@ void drumkv1widget_jack::hideEvent ( QHideEvent *pHideEvent )
 	if (m_pNsmClient)
 		m_pNsmClient->visible(false);
 
-	QWidget::hideEvent(pHideEvent);
+	drumkv1widget::hideEvent(pHideEvent);
 }
 
 #endif	// CONFIG_NSM
