@@ -1,7 +1,7 @@
 // drumkv1widget_lv2.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2018, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -62,6 +62,9 @@ protected:
 	// Param port method.
 	void updateParam(drumkv1::ParamIndex index, float fValue) const;
 
+	// Show event handler.
+	void showEvent(QShowEvent *pShowEvent);
+
 	// Close event handler.
 	void closeEvent(QCloseEvent *pCloseEvent);
 
@@ -76,6 +79,8 @@ private:
 #ifdef CONFIG_LV2_UI_IDLE
 	bool m_bIdleClosed;
 #endif
+
+	int m_iShowEvent;
 };
 
 
