@@ -1,7 +1,7 @@
 // drumkv1widget_param.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@
 #include <QDial>
 #include <QDoubleSpinBox>
 #include <QButtonGroup>
+#include <QGroupBox>
 
 
 // Forward declarations.
@@ -34,52 +35,6 @@ class QComboBox;
 class QCheckBox;
 
 
-//-------------------------------------------------------------------------
-// drumkv1widget_dial - A better QDial widget.
-
-class drumkv1widget_dial : public QDial
-{
-	Q_OBJECT
-
-public:
-
-	// Constructor.
-	drumkv1widget_dial(QWidget *pParent = 0);
-
-	// Dial mode behavior:
-	// DefaultMode - default (old) QDial behavior.
-	// LinearMode  - proportionally to distance in one ortogonal axis.
-	// AngularMode - angularly relative to widget center.
-	enum DialMode { DefaultMode = 0, LinearMode, AngularMode };
-
-	// Set knob dial mode behavior.
-	static void setDialMode(DialMode dialMode);
-	static DialMode dialMode();
-
-protected:
-
-	// Mouse angle determination.
-	float mouseAngle(const QPoint& pos);
-
-	// Alternate mouse behavior event handlers.
-	void mousePressEvent(QMouseEvent *pMouseEvent);
-	void mouseMoveEvent(QMouseEvent *pMouseEvent);
-	void mouseReleaseEvent(QMouseEvent *pMouseEvent);
-
-private:
-
-	// Alternate mouse behavior tracking.
-	bool   m_bMousePressed;
-	QPoint m_posMouse;
-
-	// Just for more precission on the movement
-	float m_fLastDragValue;
-
-	// Knob dial mode behavior.
-	static DialMode g_dialMode;
-};
-
-
 //-------------------------------------------------------------------------
 // drumkv1widget_param - Custom composite widget.
 
@@ -151,6 +106,52 @@ private:
 };
 
 
+//-------------------------------------------------------------------------
+// drumkv1widget_dial - A better QDial widget.
+
+class drumkv1widget_dial : public QDial
+{
+	Q_OBJECT
+
+public:
+
+	// Constructor.
+	drumkv1widget_dial(QWidget *pParent = 0);
+
+	// Dial mode behavior:
+	// DefaultMode - default (old) QDial behavior.
+	// LinearMode  - proportionally to distance in one ortogonal axis.
+	// AngularMode - angularly relative to widget center.
+	enum DialMode { DefaultMode = 0, LinearMode, AngularMode };
+
+	// Set knob dial mode behavior.
+	static void setDialMode(DialMode dialMode);
+	static DialMode dialMode();
+
+protected:
+
+	// Mouse angle determination.
+	float mouseAngle(const QPoint& pos);
+
+	// Alternate mouse behavior event handlers.
+	void mousePressEvent(QMouseEvent *pMouseEvent);
+	void mouseMoveEvent(QMouseEvent *pMouseEvent);
+	void mouseReleaseEvent(QMouseEvent *pMouseEvent);
+
+private:
+
+	// Alternate mouse behavior tracking.
+	bool   m_bMousePressed;
+	QPoint m_posMouse;
+
+	// Just for more precission on the movement
+	float m_fLastDragValue;
+
+	// Knob dial mode behavior.
+	static DialMode g_dialMode;
+};
+
+
 //-------------------------------------------------------------------------
 // drumkv1widget_knob - Custom knob/dial widget.
 
@@ -228,6 +229,11 @@ signals:
 	// Alternate value change signal.
 	void valueChangedEx(double);
 
+protected:
+
+	// Inherited/override methods.
+	QValidator::State validate(QString& sText, int& iPos) const;
+
 private:
 
 	// Alternate edit behavior tracking.
@@ -403,6 +409,40 @@ private:
 };
 
 
+//-------------------------------------------------------------------------
+// drumkv1widget_group - Custom checkable group-box widget.
+
+class drumkv1widget_group : public QGroupBox
+{
+	Q_OBJECT
+
+public:
+
+	// Constructor.
+	drumkv1widget_group(QWidget *pParent = 0);
+
+	// Desstructor.
+	~drumkv1widget_group();
+
+	// Accessors.
+	void setToolTip(const QString& sToolTip);
+	QString toolTip() const;
+
+	drumkv1widget_param *param() const;
+
+protected slots:
+
+	// Change slot.
+	void paramValueChanged(float);
+	void groupBoxValueChanged(bool);
+
+private:
+
+	// Widget members.
+	drumkv1widget_param *m_pParam;
+};
+
+
 #endif  // __drumkv1widget_param_h
 
 // end of drumkv1widget_param.h
